@@ -25,6 +25,9 @@
 /* Defined when the system has a working shl_load () call instead. */
 #undef USE_SHL_LOAD
 
+/* Defined when the system has cbrt() in libm. */
+#undef HAVE_CBRT
+
 @BOTTOM@
 #if defined (Solaris)
 #  include <arch/solaris/prototypes.h>
