@@ -119,7 +119,6 @@
     "sql::database-column-info" "sql::database-columns-info"
     "sql::database-query-info" "sql::database-set-options"
     "sql::cursor-get-column" "sql::query-get-column" "sql::sql-transact"
-    "tdbox"
     "example" "simple-fundoc" "complex-fundoc" "prog" "server-push"))
 
 (defun mhtml-build-font-lock-keywords (from-list)
@@ -231,7 +230,8 @@ rigidly along with this one."
      "layout::page" "layout::left-border" "layout::right-border"
      "frameset" "msql::with-open-database" "odbc::with-open-database"
      "sql::with-open-database" "dbm::with-open-database" "def-pagelist"
-     "defweakmacro" "with-safe-environment" "tab-menu" "tdbox")
+     "defweakmacro" "with-safe-environment" "tab-menu" "tdbox" "description"
+     "tdbox" "config::information-item")
   "List of tag starts which are complex.")
 
 (defconst mhtml-complex-tag-start-regex
