@@ -301,7 +301,18 @@ index.html\nhome.html");
   {
     char *key = pagefunc_get_variable ("mhtml::activation-key");
     if (check_activation_key (key) == 0)
-      exit (1);
+      {
+	if (nph)
+	  fprintf (stdout, "HTTP/1.0 200 Found\n");
+	fprintf (stdout, "Content-type: text/html\n");
+	fprintf (stdout, "<html>\n<body bgcolor=white text=red>\n");
+	fprintf (stdout, "<h3>You don't have a valid activation key!</h3>\n");
+	fprintf (stdout, "<p><b>Please contact ");
+	fprintf (stdout, "<a href=\"mailto:info@metahtml.com\">");
+	fprintf (stdout, "info@metahtml.com</a> in order to obtain one.</b>");
+	fprintf (stdout, "<p>\n</body>\n</html>\n");
+	exit (1);
+      }
     symbol_free (symbol_remove ("mhtml::activation-key"));
   }
 #endif /* CHECK_ACTIVATION */
@@ -786,6 +797,9 @@ process_request (void)
   /* Okay, read the HTTP request and handle it. */
   req = engine_make_request ();
 
+  if (mhttpd_debugging != 0)
+    mhttpd_debug_request (req);
+
 #if defined (BUILDING_WITH_FAST_CGI)
   mhtml_stdout_fileno = 1;
 #endif
@@ -804,15 +818,26 @@ process_request (void)
   if (req && mhttpd_per_request_function)
     {
       PAGE *perfun = page_create_page ();
+
+      if (mhttpd_debugging != 0)
+	mhttpd_log (log_DEBUG, "mhttpd::per-request-function (%s)",
+		    mhttpd_per_request_function);
+
       bprintf (perfun, "<%s>\n", mhttpd_per_request_function);
       page_process_page (perfun);
 
       if (mhttpd_page_redirect_p (perfun))
 	{
+	  if (mhttpd_debugging != 0)
+	    mhttpd_log (log_DEBUG, "Per request function redirected!");
+
 	  if (nph)
 	    {
-	      if (!strncasecmp (perfun->buffer, "HTTP/", 5) == 0)
-		bprintf_insert (perfun, 0, "HTTP/1.0 302 Found\n");
+	      if ((strncasecmp (perfun->buffer, "HTTP/", 5) != 0) &&
+		  (strncasecmp (perfun->buffer, "HTTPS/", 6) != 0))
+		bprintf_insert (perfun, 0, "%s/%s 302 Found\n",
+				req->protocol ? req->protocol : "HTTP",
+				req->protocol_version ? req->protocol : "1.0");
 	    }
 
 	  fwrite (perfun->buffer, perfun->bindex, 1, stdout);
