@@ -379,7 +379,8 @@ and, a subsequent execution of
   xfree (start_var);
 }
 
-DEFMACRO (pf_foreach, elementvar arrayvar &key start=x end=x step=x iter=var,
+DEFMACRO (pf_foreach, elementvar arrayvar
+	  &key start=x end=x step=x iter=var no-copy=true,
 "Perform <var body> with <var elementvar> bound to successive memebers
 of <var arrayvar>, starting with the element at <var start> (default
 0), and ending at <var end> (default <example code><array-size
@@ -389,6 +390,10 @@ The <code>foreach</code> command is the basic array looping device in
 <Meta-HTML>.  It is guaranteed to iterate over each element that you
 specify, whether that element is the empty string or not.
 
+If <var no-copy=true> is specified, the array is not copied before
+iteration, so that changes that you make to the array take place
+immediately, during the execution of the surrounding <tag foreach>.
+
 Starting with the simple array:
 <set-var array[]=\"0\\n1\\n2\\n3\\n4\\n5\\n6\\n7\\n8\\n9\">
 <example>
@@ -414,6 +419,8 @@ or, we can produce a \"countdown\" with a negative value for <var step>:
   char *end_arg = mhtml_evaluate_string (get_value (vars, "end"));
   char *step_arg = mhtml_evaluate_string (get_value (vars, "step"));
   char *iter_var = mhtml_evaluate_string (get_value (vars, "iter"));
+  char *copy = mhtml_evaluate_string (get_value (vars, "copy"));
+  char *no_copy = mhtml_evaluate_string (get_value (vars, "no-copy"));
   int start_index, end_index, step;
 
   if ((!empty_string_p (element_var)) && (!empty_string_p (array_var)))
@@ -467,9 +474,27 @@ or, we can produce a \"countdown\" with a negative value for <var step>:
 	  if (((step < 0) && (end_index < start_index)) ||
 	      ((step > 0) && (end_index > start_index)))
 	    {
-	      /* Now copy the array, so that functions which delete or
+	      /* Maybe copy the array, so that functions which delete or
 		 manipulate the contents of it do not disturb our pointers. */
-	      char **array_values = symbol_copy_array (array->values);
+	      int copyit = 1;
+	      char **array_values;
+
+	      if ((empty_string_p (copy)) && (empty_string_p (no_copy)))
+		{
+		  /* Set the default behavior. */
+		  if (var_present_p (vars, "copy"))
+		    copyit = 0;
+		  else if (var_present_p (vars, "no-copy"))
+		    copyit = 1;
+		}
+	      else if (!empty_string_p (no_copy))
+		copyit = 0;
+
+
+	      if (copyit)
+		array_values = symbol_copy_array (array->values);
+	      else
+		array_values = array->values;
 
 	      for (i = start_index;
 		   (((start_index < end_index) && (i < end_index)) ||
@@ -506,7 +531,8 @@ or, we can produce a \"countdown\" with a negative value for <var step>:
 		    }
 		}
 
-	      symbol_free_array (array_values);
+	      if (copyit)
+		symbol_free_array (array_values);
 	    }
 
 	  *newstart = start;
@@ -519,6 +545,8 @@ or, we can produce a \"countdown\" with a negative value for <var step>:
   xfree (end_arg);
   xfree (step_arg);
   xfree (iter_var);
+  xfree (no_copy);
+  xfree (copy);
 }
 
 DEFUN (pf_array_concat, receiver &optional contributor...,
