@@ -20,6 +20,8 @@ bootstrap_metahtml (int call_initializer_p)
   if (bootstrap_code_len)
     {
       int sd = symdump_open_string_data ();
+      void *ignore = (void *)xmalloc (4 * bootstrap_code_len);
+      free (ignore);
 
       if (sd != -1)
 	{
@@ -32,7 +34,8 @@ bootstrap_metahtml (int call_initializer_p)
 	  symdump_set_string_data_buffer_size (sd, bootstrap_code_len);
 	  while ((pack = symbol_load_package (sd)) != (Package *)NULL);
 	  if (mhtml_user_keywords == (Package *)NULL)
-	    mhtml_user_keywords = symbol_lookup_package ("*user-functions*");
+	    mhtml_user_keywords =
+	      symbol_get_package_hash ("*user-functions*", 577);
 	  bprintf_free_buffer (symdump_close_string_data (sd));
 
 	  /* Always call <bootstrapper::system-initialize>. */
