@@ -60,7 +60,25 @@ static PFunDesc func_table[] =
 };
 
 PACKAGE_INITIALIZER (initialize_db_functions)
-DOC_SECTION (DATABASE-COMMANDS)
+DEFINE_SECTION (DATABASE-COMMANDS, database; long-term storage, 
+"<meta-html> contains commands for performing operations on a
+database. A database created by <meta-html> contains <i>records</i>
+stored by <i>key</i>, where each record consists of a set of
+<i>name/value</i> pairs.
+
+There is a single command for specifying which database you will be
+operating on: <funref database-commands with-open-database>.  All of
+the remaining database commands only have an effect when executed
+within the scope of this function.
+
+<funref \"database commands\" database-query> is the command used to
+perform queries on a database, or to simply select a range of records
+to operate on.
+
+Functions which load, store, or delete a record as a single atomic
+operation, return the string <code>\"true\"</code> when they succeed,
+and store an error message in <funref Language-Operators
+SYSTEM-ERROR-OUTPUT> when they do not.", "")
 
 static long
 random_number (void)
@@ -229,7 +247,8 @@ database.  If the database does not exist, it is created.
   int jump_again = 0;
 
   varname = mhtml_evaluate_string (get_positional_arg (vars, 0));
-  dbname = mhtml_canonicalize_file_name_argument(get_positional_arg (vars, 1));
+  dbname = mhtml_canonicalize_file_name_argument 
+    (get_positional_arg (vars, 1));
 
   if ((!empty_string_p (varname)) && (!empty_string_p (dbname)))
     {
@@ -418,7 +437,7 @@ Upon success, this function returns the string <code>\"true\"</code>.
 	    }
 	  else
 	    {
-	      register which = 0;
+	      register int which = 0;
 
 	      if ((name != (char *)NULL) && (name[0] != '\0'))
 		{
@@ -532,7 +551,8 @@ Also see <funref database-commands database-save-package>, and
 
       bprintf (data, "(");
 
-      while ((name = get_positional_arg (vars, position)) != (char *)NULL)
+      while ((name = mhtml_evaluate_string
+	      (get_positional_arg (vars, position))) != (char *)NULL)
 	{
 	  char *value = pagefunc_get_variable (name);
 
@@ -551,6 +571,7 @@ Also see <funref database-commands database-save-package>, and
 	    bprintf (data, "(%s . ", wisp_readable (name));
 
 	  bprintf (data, "%s)", value ? wisp_readable (value) : "\"\"");
+	  free (name);
 	}
 
       bprintf (data, ")");
@@ -587,7 +608,7 @@ written.
 Upon success, this function returns the string <code>true</code>.
 
 <example>
-<set-var mykey="The Key" foo::x=x foo::array[0]=val0 foo::array[1]=val1>
+<set-var mykey=\"The Key\" foo::x=x foo::array[0]=val0 foo::array[1]=val1>
 <with-open-database db /tmp/file.db mode=write-create>
   <database-save-package db <get-var mykey> foo strip=true>
 </with-open-database>
@@ -689,7 +710,7 @@ true
 }
 
 DEFUN (pf_database_first_key, dbvar,
-"Return a string representing the "first" key found in the database
+"Return a string representing the \"first\" key found in the database
 referenced by <var dbvar>.  The key is suitable for input to any of
 the database functions which takes a key as input.  The order in which
 keys are returned appears random, but the return value is always a
