@@ -62,7 +62,14 @@ static PFunDesc func_table[] =
 };
 
 PACKAGE_INITIALIZER (initialize_dbm_functions)
-DOC_SECTION (direct-dbm-access)
+DEFINE_SECTION (DIRECT-DBM-ACCESS, database; perl, 
+"There are times when it is desirable to directly access DBM, NDBM, or
+GDBM databases, for example, when reading or writing values to a
+database created with a different tool, such as Perl or TCL.
+
+<Meta-HTML> provides low-level access primitives for such databases in
+addition to a higher-level, more generally useful abstraction
+(which is detailed in <secref DATABASE-COMMANDS>).", "")
 
 /************************************************************/
 /*							    */
