@@ -252,6 +252,7 @@ dsn_lookup (char *key, char *str)
 #endif
 
 #if defined (COMPILING_MYSQLFUNCS)
+#  define CACHED_GSQL_LIST_FIELDS
 #  include "mysqlfuncs.c"
 #endif
 
@@ -260,6 +261,7 @@ dsn_lookup (char *key, char *str)
 #endif
 
 #if defined (COMPILING_ODBCFUNCS)
+#  define CACHED_GSQL_LIST_FIELDS
 #  include "odbcfuncs.c"
 #endif
 
@@ -558,7 +560,7 @@ lock_database (char *dsn, int *lock, Database *db, int lock_p)
 static void
 unlock_database (char *dsn, int *lock, Database *db, int lock_p)
 {
-  if (db != NULL)
+  if (db != (Database *)NULL)
     gsql_close (db);
 
   if (lock_p && (*lock > -1))
@@ -648,8 +650,7 @@ pf_with_open_database (PFunArgs)
     }
 
   /* If the user supplied a DSN, use it, otherwise make one out of
-   the hostname and dbname
-   */
+     the hostname and dbname. */
 
   if ((empty_string_p (dsn)) && (!empty_string_p (dbname)))
     {
@@ -769,13 +770,13 @@ pf_database_exec_sql (PFunArgs)
 
 	      query_result = gsql_store_result (db);
 
-	      if (query_result != (gsql_result *) NULL)
+	      if (query_result != (gsql_result *)NULL)
 		{
 		  gsql_fetch_row (query_result);
 		  gsql_free_result (query_result);
 		}
 	      
-	      free (dbquery);
+	      xfree (dbquery);
 	    }
 	}
     }
@@ -871,7 +872,7 @@ pf_database_exec_query (PFunArgs)
 		  xfree (cursorvar);
 		}
 
-	      free (dbquery);
+	      xfree (dbquery);
 	    }
 	}
     }
@@ -1518,6 +1519,10 @@ pf_database_columns (PFunArgs)
 		    }
 
 		  xfree (varname);
+
+#if !defined (CACHED_GSQL_LIST_FIELDS)
+		  gsql_free_result (result);
+#endif
 		}
 	    }
 	  xfree (tablename);
@@ -1542,14 +1547,13 @@ package_from_row (gsql_result *query_result,
 {
   register int i;
   int num_fields = 0;
-  Package *package = symbol_get_package ((char *)NULL);
+  Package *package = (Package *)NULL;
   char *prev_fieldname = (char *) NULL;
   char *fieldname = (char *) NULL;
 
   /* Loop over fields of the row, binding variables.  All results
      should already be strings, so we don't need to worry about
      converting INT, REAL, or other datatypes to strings. */
-
   num_fields = gsql_num_fields (query_result);
 
   for (i = 0; i < num_fields; i++)
@@ -1595,6 +1599,7 @@ package_from_row (gsql_result *query_result,
 	    }
 
 	  value = gsql_get_column (query_result, i);
+	  if (!package) package = symbol_get_package ((char *)NULL);
 	  sym = symbol_intern_in_package (package, buf);
 	  symbol_add_value (sym, value ? value : "");
 	}
@@ -1711,7 +1716,7 @@ pf_database_query (PFunArgs)
 	  while (rec_count < window_limit)
 	    {
 	      int status = gsql_fetch_row (query_result);
-	      Package *db_fields;
+	      Package *db_fields = (Package *)NULL;
 
 	      /* If at end of data, we're done with this loop. */
 	      if ((status != GSQL_SUCCESS) &&
@@ -1767,7 +1772,6 @@ pf_database_query (PFunArgs)
 		    }
 		  else
 		    symbol_destroy_package (db_fields);
-
 		  xfree (expr_result);
 		}
 	    }
@@ -2252,6 +2256,11 @@ pf_database_save_record (PFunArgs)
 	}
     }
 
+#if !defined (CACHED_GSQL_LIST_FIELDS)
+  if (table_fields != (gsql_result *)NULL)
+    gsql_free_result (table_fields);
+#endif
+
   xfree (key);
   xfree (keyname);
   xfree (method);
@@ -2299,7 +2308,7 @@ pf_database_load_record (PFunArgs)
       int truncate_p = gsql_database_sql_truncate_columns (db);
 
       pkgname = mhtml_evaluate_string
-	(get_one_of (vars, "PACKAGE", "PREFIX", (char *)NULL));
+	(get_one_of (vars, "PACK", "PACKAGE", "PREFIX", (char *)NULL));
 
       table_fields = gsql_list_fields (db, table);
 
@@ -2388,6 +2397,11 @@ pf_database_load_record (PFunArgs)
 	}
     }
 
+#if !defined (CACHED_GSQL_LIST_FIELDS)
+  if (table_fields != (gsql_result *)NULL)
+    gsql_free_result (table_fields);
+#endif
+
   xfree (key);
   xfree (keyname);
   xfree (table);
@@ -2475,6 +2489,11 @@ pf_database_delete_record (PFunArgs)
 	gsql_save_error_message (db, GSQL_DEFAULT_ERRMSG);
     }
 
+#if !defined (CACHED_GSQL_LIST_FIELDS)
+  if (table_fields != (gsql_result *)NULL)
+    gsql_free_result (table_fields);
+#endif
+
   xfree (key);
   xfree (keyname);
   xfree (table);
@@ -2653,7 +2672,9 @@ pf_database_save_package (PFunArgs)
 			  BPRINTF_BUFFER *tmpbuf = bprintf_create_buffer ();
 
 			  for (j = 0; j < sym->values_index; j++)
-			    bprintf (tmpbuf, "%s", sym->values[j]);
+			    bprintf (tmpbuf, "%s%s",
+				     sym->values[j],
+				     sym->values[j + 1] ? " " : "");
 
 			  bprintf_gsql_escape (query, tmpbuf->buffer, field,
 					       escape_char, truncate_p);
@@ -2734,7 +2755,8 @@ pf_database_save_package (PFunArgs)
 			  BPRINTF_BUFFER *tmpbuf = bprintf_create_buffer ();
 
 			  for (j = 0; j < sym->values_index; j++)
-			    bprintf (tmpbuf, "%s", sym->values[j]);
+			    bprintf (tmpbuf, "%s%s", sym->values[j],
+				     sym->values[j + 1] ? " " : "");
 
 			  bprintf_gsql_escape (query, tmpbuf->buffer, field,
 					       escape_char, truncate_p);
@@ -2771,8 +2793,14 @@ pf_database_save_package (PFunArgs)
 	  bprintf_insert (page, start, "%s", result);
 	  *newstart += strlen (result);
 	}
+
     }
 
+#if !defined (CACHED_GSQL_LIST_FIELDS)
+  if (table_fields != (gsql_result *)NULL)
+    gsql_free_result (table_fields);
+#endif
+
   xfree (symbols);
   xfree (key);
   xfree (keyname);
@@ -2937,6 +2965,9 @@ pf_database_columns_info (PFunArgs)
 		    }
 
 		  xfree (varname);
+#if !defined (CACHED_GSQL_LIST_FIELDS)
+		  gsql_free_result (result);
+#endif
 		}
 	    }
 	  xfree (tablename);
@@ -3002,6 +3033,9 @@ pf_database_column_info (PFunArgs)
 		      *newstart += strlen (alist);
 		      free (alist);
 		    }
+#if !defined (CACHED_GSQL_LIST_FIELDS)
+		  gsql_free_result (result);
+#endif
 		}
 	    }
 	  xfree (tablename);
