@@ -42,7 +42,6 @@ static void pf_define_tag (PFunArgs);
 static void pf_define_function (PFunArgs);
 static void pf_defweakmacro (PFunArgs);
 static void pf_undef (PFunArgs);
-static void pf_function_def (PFunArgs);
 
 /************************************************************/
 /*							    */
@@ -60,13 +59,158 @@ static PFunDesc func_table[] =
   { "DEFUN",		1, 0, pf_define_function },
   { "DEFWEAKMACRO",	1, 0, pf_defweakmacro },
   { "UNDEF",		0, 0, pf_undef },
-  { "FUNCTION-DEF",	0, 0, pf_function_def },
 
   { (char *)NULL,	0, 0, (PFunHandler *)NULL }
 };
 
 PACKAGE_INITIALIZER (initialize_macro_functions)
-DOC_SECTION (MACRO-COMMANDS)
+DEFINE_SECTION (MACRO-COMMANDS, macros; functions; new operators, 
+"<meta-html> contains a powerful <i>macro</i> facility, which allows
+you to define your own commands.  Such commands are first-class
+objects in <meta-html>; they may even supersede the compiled in
+definitions.
+
+There are two types of macros that you can define.  One type is a
+<i>complex-tag</i>; it consists of an opening tag, a body, and a
+closing tag.  The other type is a <i>simple-tag</i>; it only has an
+opening tag.
+
+You create a macro by using one of the macro-defining commands.  In
+the body of the definition, special keywords can be placed, which
+affect what is produced when the macro is invoked.  As a macro writer,
+you have logical access to the arguments passed to the macro in the
+opening tag, and for complex-tags, you have access to the body which
+appears between the opening and closing tags.  ",
+
+"In the opening tag of the macro defining command, several special
+<i>meta-arguments</i> may be used to affect the binding method used at
+invocation time to bind the passed parameters to the formal arguments.
+
+<ul> <li> <b>&optional</b><br> Indicates that the following named
+parameter is optional, and does not have to be supplied.  While at
+this time Meta-HTML does not complain if there are missing arguments
+at invocation time, it is likely that the byte-compiler will require
+function calls to match the formal parameters of the defined function.
+
+<example>
+<defun func x &optional y> <get-var x>, <get-var y> </defun>
+</example>
+
+<li> <b>&key</b><br> Indicates that the following named parameters
+will be bound by the caller placing the name of the parameter followed
+by an equals sign, and the value of that parameter in the opening tag
+of the function call.  Thus, keyword arguments may appear in any order
+in the calling function.  Here is an example of defining a tag called
+<tag image> which will add the width and hieght if they are not
+already present:
+
+<example>
+<defun image &key src width height>
+  <if <or <not <get-var width>>
+	  <not <get-var height>>>
+      <find-image-xy <get-var src> width height>>
+  <img src=\"<get-var src>\" width=<get-var width> height=<get-var height>>
+</defun>
+</example>
+
+<li> <b>&rest</b><br> Gobbles up any remaining arguments to the
+function, collecting them in the named parameter which follows the
+<i>&rest</i>.  The arguments may be gathered into a single string, or
+into an array, with one argument per slot.  This is controlled by
+writing the formal parameter name either with or without sqaure
+braces: (i.e., <code>foo[]</code> or <code>foo</code>).
+
+<example>
+<defun func req-arg &rest rest-args[]>
+  <ol>
+    <foreach x rest-args>
+      <li> <get-var x> </li>
+    </foreach>
+  </ol>
+</defun>
+</example>
+
+<li> <b>&body</b><br> Causes the following named parameter to be bound
+to the body of the invoked function or macro.  For <tag defun> and
+<tag defsubst>, this is all of the material which appeared in the
+opening tag, while for <tag defmacro> and <tag defweakmacro>, this is
+all of the material that appeared between the opening and closing
+tags.
+
+<example>
+<defmacro with-debugging-output &body body>
+  <with-open-stream debug-stream /tmp/debug-output mode=append>
+    <stream-put debug-stream <get-var body>>
+  </with-open-stream>
+</defmacro>
+</example>
+
+<li> <b>&unevalled</b><br> Modifies the binding rule of a formal
+parameter such that the material which is bound is not evaluated
+before the binding takes place.  This is almost equivalent to using
+the <b>%0</b> ... <b>%9</b>, or <b>%body</b> textual substitutions,
+but the arguments are bound to variables instead of pure textual
+substitution.  Here is how one might write a function which takes an
+expression, and produces the expression and the evaluation of the
+expression as output:
+
+<example>
+<defun debug-expr &body &unevalled qbody &body body>
+  <get-var-once qbody> EVALS TO: <get-var-once body>
+</defun>
+</example>
+
+Such an invocation might look like:
+<example>
+  <set-var x=4 y=5>
+  <debug-expr <add x y>>
+</example>
+
+which would produce:
+<example>
+  <add x y> EVALS TO: 9
+</example>
+</ul>
+
+Here is a ridiculous function, which uses all of the special
+meta-parameters:
+<example>
+<defsubst func req &optional opt &key k1 &unevalled k2 &body b &rest args[]>
+   REQ: <get-var-once req>,
+   OPT: <get-var-once opt>
+    K1: <get-var-once k1>
+    K2: <get-var-once k2>
+  BODY: <get-var-once b>
+  REST: <foreach arg args><get-var-once arg> </foreach>
+</defsubst>
+</example>
+
+And, here are examples of calling that function:
+
+Example 1:
+<example>
+<set-var key-1-arg=key-1>
+<func required k2=\"Unevalled\" opt-arg k1=<get-var key-1-arg> rest0 rest1>
+   REQ: required,
+   OPT: opt-arg
+    K1: key-1
+    K2: Unevalled
+  BODY: required k2=\"Unevalled\" opt-arg k1=key-1 rest0 rest1
+  REST: rest0 rest1
+</example>
+Example 2:
+<example>
+<func k2=<get-var k1> required rest0 rest1>
+   REQ: required,
+   OPT: rest0
+    K1: 
+    K2: <get-var k1>
+  BODY: k2= required rest0 rest1
+  REST: rest1
+</example>
+
+Notice how in the second example, our optional parameter <b>opt</b>
+got bound to the second non-keyword argument <code>rest0</code>!")
 
 DEFMACROX (pf_defsubst, name &optional named-parameters &key
 	  package=packname whitespace=delete,
@@ -273,9 +417,9 @@ name> that has been defined in this way, the definition is removed.")
 
 	  if (uf)
 	    {
-	      free (uf->body);
-	      free (uf->name);
-	      free (uf);
+	      make_gcable ((char *)uf->body);
+	      make_gcable ((char *)uf->name);
+	      make_gcable ((char *)uf);
 	    }
 
 	  sym = symbol_remove_in_package (mhtml_user_keywords, varname);
@@ -292,93 +436,6 @@ name> that has been defined in this way, the definition is removed.")
     }
 }
 
-DEFUN (pf_function_def, funname,
-"Return a human readable rendition of the function named by <var funname>.")
-{
-  if (mhtml_user_keywords == (Package *)NULL)
-    return;
-  else
-    {
-      register int i;
-      char *name;
-    
-      for (i = 0; (name = get_positional_arg (vars, i)) != (char *)NULL; i++)
-	{
-	  char *varname = mhtml_evaluate_string (name);
-
-	  if (varname != (char *)NULL)
-	    {
-	      UserFunction *uf = mhtml_find_user_function (varname);
-
-	      if (uf)
-		{
-		  BPRINTF_BUFFER *insertion = bprintf_create_buffer ();
-
-		  if (uf->type == user_MACRO)
-		    {
-		      if (uf->flags & user_WEAK_MACRO)
-			bprintf (insertion, "<defweakmacro %s", uf->name);
-		      else
-			bprintf (insertion, "<define-container %s", uf->name);
-		    }
-		  else if (uf->type == user_DEFUN)
-		    bprintf (insertion, "<define-function %s", uf->name);
-		  else if (uf->type == user_SUBST)
-		    bprintf (insertion, "<define-tag %s", uf->name);
-
-		  /* Any named parameters? */
-		  if (uf->named_parameters != (char **)NULL)
-		    {
-		      register int j;
-
-		      for (j = 0; uf->named_parameters[j] != (char *)NULL; j++)
-			bprintf (insertion, " %s", uf->named_parameters[j]);
-		    }
-
-		  /* Any special package name? */
-		  if (uf->packname != (char *)NULL)
-		    {
-		      if ((uf->type != user_DEFUN) ||
-			  (strcmp (uf->packname, "local") != 0))
-			bprintf (insertion, " package=%s", uf->packname);
-		    }
-
-		  /* Any CR's in body? */
-		  if ((uf->type != user_DEFUN) &&
-		      (uf->flags & user_WHITESPACE_DELETED))
-		    bprintf (insertion, " whitespace=delete>\n");
-		  else
-		    bprintf (insertion, ">\n");
-
-		  bprintf (insertion, "%s", uf->body);
-
-		  bprintf (insertion, "\n");
-
-		  if (uf->type == user_MACRO)
-		    {
-		      if (uf->flags & user_WEAK_MACRO)
-			bprintf (insertion, "</defweakmacro>");
-		      else
-			bprintf (insertion, "</define-container>");
-		    }
-		  else if (uf->type == user_DEFUN)
-		    bprintf (insertion, "</define-function>");
-		  else if (uf->type == user_SUBST)
-		    bprintf (insertion, "</define-tag>");
-
-		  bprintf_insert (page, start, "%s\n", insertion->buffer);
-		  start += insertion->bindex + 1;
-		  bprintf_free_buffer (insertion);
-		}
-
-	      free (varname);
-	    }
-	}
-
-      *newstart = start;
-    }
-}
-
 #if defined (__cplusplus)
 }
 #endif
