@@ -149,6 +149,13 @@ typedef struct
   char *dbname;
   char *hostname;
 
+#if defined (CACHED_GSQL_LIST_FIELDS)
+  /* Cache the saved information about fields of tables */
+  void **table_cache_info;	/* An array of gsql_result. */
+  int table_cache_index;
+  int table_cache_slots;
+#endif
+
 } Database;
 
 typedef struct
@@ -170,6 +177,7 @@ typedef struct
   MYSQL_RES *mysql_result;
   MYSQL_ROW mysql_row;
   gsql_field **fields;
+  char *tablename;
 } gsql_result;
 
 
@@ -184,13 +192,35 @@ typedef struct
 
 static void free_database_cursors (Database *db);
 
+#if defined (CACHED_GSQL_LIST_FIELDS)
+static void gsql_free_result (gsql_result *result);
+
+static void
+free_cached_table_info (Database *db) 
+{ 
+  if ((db != (Database *)NULL) && (db->table_cache_info != (void **)NULL))
+    {
+      register int i;
+
+      for (i = 0; db->table_cache_info[i] != (void *)NULL; i++)
+	gsql_free_result ((gsql_result *)db->table_cache_info[i]);
+
+      free (db->table_cache_info);
+    }
+}
+#endif
+
 static void
 free_database_resources (Database *db)
 {
-  /* Free cursors, namestrings, hdbc resources, etc */
+  /* Free cursors, namestrings, hdbc resources, etc. */
   if (db != (Database *)NULL)
     {
       free_database_cursors (db);
+#if defined (CACHED_GSQL_LIST_FIELDS)
+      /* Free cached table information. */
+      free_cached_table_info (db);
+#endif
       xfree (db->dbname);
       xfree (db->hostname);
       db->dbname = (char *)NULL;
@@ -318,6 +348,8 @@ gsql_free_result (gsql_result *result)
       free (result->fields);
     }
 
+  xfree (result->tablename);
+
   if (mr != (MYSQL_RES *)NULL) mysql_free_result (mr);
 
   free (result);
@@ -330,11 +362,48 @@ initialize_database (Database *db)
   db->hostname = (char *) NULL;
   db->sock= (MYSQL *) NULL;
 
+#if defined (CACHED_GSQL_LIST_FIELDS)
+  db->table_cache_index = 0;
+  db->table_cache_slots = 0;
+  db->table_cache_info = (void **)NULL;
+#endif
+
   db->sql_escape_char       = DEFAULT_SQL_ESCAPE_CHARACTER;
   db->sql_truncate_columns  = DEFAULT_SQL_TRUNCATE_COLUMNS;
   db->sql_prefix_tablenames = DEFAULT_SQL_PREFIX_TABLENAMES;
 }
 
+#if defined (CACHED_GSQL_LIST_FIELDS)
+static void
+database_add_cached_table_info (Database *db, gsql_result *gr)
+{
+  if (db->table_cache_index + 2 > db->table_cache_slots)
+    db->table_cache_info = (void **)xrealloc
+      (db->table_cache_info,
+       ((db->table_cache_slots += 10) * sizeof (gsql_result *)));
+
+  db->table_cache_info[db->table_cache_index++] = (void *)gr;
+  db->table_cache_info[db->table_cache_index] = (void *)NULL;
+}
+
+/* Search for the first result with a matching tablename.  */
+static gsql_result *
+database_lookup_cached_table_info (Database *db, char *tablename)
+{
+  int i;
+
+  for (i = 0; i < db->table_cache_index; i++)
+    {
+      gsql_result *gr = (gsql_result *)db->table_cache_info[i];
+      if ((gr->tablename != (char *) NULL) &&
+	  (strcasecmp (tablename, gr->tablename) == 0))
+	return (gr);
+    }
+
+  return (gsql_result *) NULL;
+}
+#endif /* CACHED_GSQL_LIST_FIELDS */
+
 static gsql_result *
 make_gsql_result (void)
 {
@@ -344,6 +413,7 @@ make_gsql_result (void)
   g->mysql_result = (MYSQL_RES *) NULL;
   g->mysql_row = (MYSQL_ROW) NULL;
   g->fields = (gsql_field **) NULL;
+  g->tablename = (char *)NULL;
 
   return (g);
 }
@@ -460,26 +530,26 @@ gsql_field_type (gsql_field *field)
 {
   switch (field->mysql_field->type)
     {
-    case FIELD_TYPE_CHAR:	return (GSQL_CHAR);
+    case FIELD_TYPE_CHAR:		return (GSQL_CHAR);
       /*    case FIELD_TYPE_NUMERIC:	return (GSQL_NUMERIC); */
-    case FIELD_TYPE_DECIMAL:	return (GSQL_DECIMAL);
-    case FIELD_TYPE_INT24:	return (GSQL_INTEGER);
+    case FIELD_TYPE_DECIMAL:		return (GSQL_DECIMAL);
+    case FIELD_TYPE_INT24:		return (GSQL_INTEGER);
       /* case FIELD_TYPE_SMALLINT:	return (GSQL_SMALLINT); */
-    case FIELD_TYPE_FLOAT:	return (GSQL_FLOAT);
-      /* case FIELD_TYPE_REAL:	return (GSQL_REAL); */
-    case FIELD_TYPE_DOUBLE:	return (GSQL_DOUBLE);
-      /* case FIELD_TYPE_STRING:     return (GSQL_VARCHAR); */
+    case FIELD_TYPE_FLOAT:		return (GSQL_FLOAT);
+      /* case FIELD_TYPE_REAL:		return (GSQL_REAL); */
+    case FIELD_TYPE_DOUBLE:		return (GSQL_DOUBLE);
+      /* case FIELD_TYPE_STRING:	return (GSQL_VARCHAR); */
 
       /* non-standard ANSI follows */
-    case FIELD_TYPE_NULL:     return (GSQL_NULL);
-    case FIELD_TYPE_TINY_BLOB:     return (GSQL_TINY_BLOB);
-    case FIELD_TYPE_MEDIUM_BLOB:     return (GSQL_MEDIUM_BLOB);
-    case FIELD_TYPE_LONG_BLOB:     return (GSQL_LONG_BLOB);
-    case FIELD_TYPE_BLOB:     return (GSQL_BLOB);
-    case FIELD_TYPE_STRING:     return (GSQL_STRING);
-    case FIELD_TYPE_VAR_STRING:     return (GSQL_VAR_STRING);
+    case FIELD_TYPE_NULL:		return (GSQL_NULL);
+    case FIELD_TYPE_TINY_BLOB:		return (GSQL_TINY_BLOB);
+    case FIELD_TYPE_MEDIUM_BLOB:	return (GSQL_MEDIUM_BLOB);
+    case FIELD_TYPE_LONG_BLOB:		return (GSQL_LONG_BLOB);
+    case FIELD_TYPE_BLOB:		return (GSQL_BLOB);
+    case FIELD_TYPE_STRING:		return (GSQL_STRING);
+    case FIELD_TYPE_VAR_STRING:		return (GSQL_VAR_STRING);
 
-    default:		return (GSQL_ANY);
+    default:				return (GSQL_ANY);
     }
 }
 
@@ -487,13 +557,25 @@ gsql_field_type (gsql_field *field)
 static gsql_result *
 gsql_list_fields (Database *db, char *tablename)
 {
-  MYSQL_RES *mr = mysql_list_fields (db->sock, tablename, NULL);
   gsql_result *gr = (gsql_result *) NULL;
+  MYSQL_RES *mr = (MYSQL_RES *)NULL;
+
+#if defined (CACHED_GSQL_LIST_FIELDS)
+  gr = database_lookup_cached_table_info (db, tablename);
+  if (gr != (gsql_result *) NULL) return (gr);
+#endif /* CACHED_GSQL_LIST_FIELDS */
+
+  mr = mysql_list_fields (db->sock, tablename, NULL);
 
   if (mr != (MYSQL_RES *) NULL)
     {
       gr = gsql_make_field_array (mr);
+      gr->tablename = strdup (tablename);
+#if defined (CACHED_GSQL_LIST_FIELDS)
+      database_add_cached_table_info (db, gr);
+#endif
     }
+
   return (gr);
 }
 
