@@ -43,6 +43,8 @@ static void pf_in_package (PFunArgs);
 static void pf_with_local_package (PFunArgs);
 static void pf_package_to_alist (PFunArgs);
 static void pf_alist_to_package (PFunArgs);
+static void pf_alist_package_names (PFunArgs);
+static void pf_alist_package_delete (PFunArgs);
 
 /* The "convenience" functions follow. */
 static void pf_alist_get_var (PFunArgs);
@@ -67,6 +69,8 @@ static PFunDesc func_table[] =
   { "PACKAGE-TO-ALIST",		0, 0, pf_package_to_alist },
   { "ALIST-TO-PACKAGE",		0, 0, pf_alist_to_package },
 
+  { "ALIST-PACKAGE-NAMES",	0, 0, pf_alist_package_names },
+  { "ALIST-PACKAGE-DELETE",	0, 0, pf_alist_package_delete },
   { "ALIST-GET-VAR",		0, 0, pf_alist_get_var },
   { "ALIST-SET-VAR",		0, 0, pf_alist_set_var },
   { "ALIST-DEFVAR",		0, 0, pf_alist_defvar },
@@ -112,7 +116,7 @@ converting packages from an internal representation to a printable
 representation, called an <i>alist</i>.")
 
 DEFUN (pf_package_names, ,
-"Returns a newline separated list of the all of the named packages
+"Return a newline separated list of all of the named packages
 which are currently defined.  Because the list is newline separated,
 the result can easily be assigned to an array variable:
 
@@ -201,12 +205,18 @@ variable:
 		      && (strip == (char *)NULL))
 		    {
 		      bprintf_insert (page, start, "%s::%s\n",
-				      pack->name, symbols[i]->name);
+				      pack->name,
+				      symbols[i]->preserved_name ?
+				      symbols[i]->preserved_name :
+				      symbols[i]->name);
 		      start += pack->name_len + 3 + symbols[i]->name_len;
 		    }
 		  else
 		    {
-		      bprintf_insert (page, start, "%s\n", symbols[i]->name);
+		      bprintf_insert (page, start, "%s\n",
+				      symbols[i]->preserved_name ?
+				      symbols[i]->preserved_name :
+				      symbols[i]->name);
 		      start += symbols[i]->name_len + 1;
 		    }
 		}
@@ -412,9 +422,9 @@ package <code>FOO</code> into the package <code>BAR</code>:
 		}
 	      free (symbols);
 	    }
-
-	  symbol_destroy_package (from);
 	}
+
+      symbol_destroy_package (from);
     }
 
   xfree (alist);
@@ -462,7 +472,9 @@ set_alist_var (Package *vars, Package *value)
     }
 
   xfree (varname);
-  symbol_destroy_package (value);
+
+  if (value != (Package *)NULL)
+    symbol_destroy_package (value);
 }
 
 DEFUN (pf_alist_defvar, alistvar name value,
@@ -490,6 +502,8 @@ to <var value>, in the association list referenced by <alistvar>.
 
       set_alist_var (vars, alist);
     }
+  else if (alist != (Package *)NULL)
+    symbol_destroy_package (alist);
 
   xfree (name);
 }
@@ -658,8 +672,7 @@ The values are returned in the order in which the <var name>s appear.")
       if (debug_level > 5)
 	page_debug ("--> `%s'", value ? value : "");
 
-      if (insertion)
-	free (insertion);
+      xfree (insertion);
 
       *newstart = start;
     }
@@ -667,6 +680,164 @@ The values are returned in the order in which the <var name>s appear.")
   symbol_destroy_package (alist);
 }
 
+DEFUN (pf_alist_package_names, alistvar,
+"Return a newline separated list of all of the packages which are
+defined within the association list stored within <var alistvar>.
+Because the list is newline separated, the result can easily be
+assigned to an array variable:")
+{
+  Package *alist = get_alist_var (vars);
+
+  if (alist != (Package *)NULL)
+    {
+      register int i;
+      Symbol **syms = symbols_of_package (alist);
+      char **values = (char **)NULL;
+      int values_slots = 0;
+      int values_index = 0;
+
+      for (i = 0; syms[i] != (Symbol *)NULL; i++)
+	{
+	  char *name = syms[i]->name;
+	  char *dots = name ? strstr (name, "::") : (char *)NULL;
+
+	  if (dots != (char *)NULL)
+	    {
+	      register int j, found = 0;
+	      int len = (int) (dots - name);
+
+	      for (j = 0; j < values_index; j++)
+		if (strncmp (values[j], name, len) == 0)
+		  {
+		    found = 1;
+		    break;
+		  }
+
+	      if (!found)
+		{
+		  char *pack = (char *)xmalloc (1 + len);
+
+		  strncpy (pack, name, len);
+		  pack[len] = '\0';
+
+		  if (values_index + 2 > values_slots)
+		    values = (char **)xrealloc
+		      (values, (values_slots += 10) * sizeof (char *));
+
+		  values[values_index++] = pack;
+		  values[values_index] = (char *)NULL;
+		}
+	    }
+	}
+
+      if (values != (char **)NULL)
+	{
+	  for (i = 0; i < values_index; i++)
+	    {
+	      bprintf_insert (page, start, "%s\n", values[i]);
+	      start += 1 + strlen (values[i]);
+	      free (values[i]);
+	    }
+
+	  free (values);
+	  *newstart = start;
+	}
+
+      symbol_destroy_package (alist);
+    }
+}
+
+DEFUN (pf_alist_package_delete, alistvar &rest packages[],
+"For each variable in the association list within <var alistvar>, remove
+the variable from the association list if it is prefixed with one of the
+specifed package names.  For example, given that the variable <var alist>
+contained the alist:
+
+<example>
+((\"FOO::BAR\" . \"bar\") (\"FOO::BAZ\" . \"baz\") (\"BAR::X\" . \"val\"))
+</example>
+
+then calling:
+
+<example>
+<alist-package-delete alist foo>
+<get-var-once alist>
+</example>
+
+produces:
+
+<example>
+((\"BAR::X\" . \"val\"))
+</example>")
+{
+  Package *alist = get_alist_var (vars);
+
+  if (alist != (Package *)NULL)
+    {
+      Symbol **syms = symbols_of_package (alist);
+
+      if (syms != (Symbol **)NULL)
+	{
+	  register int i, j;
+	  char **packnames = (char **)NULL;
+	  int packnames_index = 0;
+	  int packnames_slots = 0;
+	  char *arg;
+
+	  /* Collect the names of the packages passed as parameters. */
+	  for (i = 1;
+	       (arg = (mhtml_evaluate_string (get_positional_arg (vars, i))))
+		 != (char *)NULL;
+	       i++)
+	    {
+	      /* Force to all upper case. */
+	      for (j = 0; arg[j] != '\0'; j++)
+		if (islower (arg[j]))
+		  arg[j] = toupper (arg[j]);
+
+	      /* Append it to our list of package names to check. */
+	      if (packnames_index + 2 > packnames_slots)
+		packnames = (char **)xrealloc
+		  (packnames, (packnames_slots += 10) * sizeof (char *));
+
+	      packnames[packnames_index++] = arg;
+	      packnames[packnames_index] = (char *)NULL;
+	    }
+
+	  /* Okay, have all of the arguments.  Now figure out which variables
+	     to remove. */
+	  for (i = 0; syms[i] != (Symbol *)NULL; i++)
+	    {
+	      char *name = syms[i]->name;
+	      char *dots = name ? strstr (name, "::") : (char *)NULL;
+
+	      if (dots != (char *)NULL)
+		{
+		  int len = (int) (dots - name);
+
+		  for (j = 0; j < packnames_index; j++)
+		    if (strncmp (packnames[j], name, len) == 0)
+		      {
+			Symbol *sym = symbol_remove_in_package (alist, name);
+			symbol_free (sym);
+		      }
+		}
+	    }
+
+	  /* All variables removed.  Replace the package. */
+	  set_alist_var (vars, alist);
+
+	  /* Free the memory used to store the package names. */
+	  for (i = 0; i < packnames_index; i++)
+	    free (packnames[i]);
+	  xfree (packnames);
+	}
+      else
+	symbol_destroy_package (alist);
+    }
+}
+
 #if defined (__cplusplus)
 }
 #endif
+
