@@ -324,16 +324,16 @@ sequence of 3 semicolon character is seen
 (<code>;</code><code>;;</code>), then the text from this sequence to 
 the end of the line inclusive is discarded.
 
-Example:
+Example:<set-var mhtml::inhibit-comment-parsing=1>
 <example>
 <comment>
   This text is simply ignored.
 
   All of it.
 </comment>
-;\;\; So is this text, up to the end of the line.
+;;; So is this text, up to the end of the line.
 ;; But NOT this line -- it only has two semi-colons on it.
-</example>")
+</example><unset-var mhtml::inhibit-comment-parsing>")
 {
   /* Contents already deleted by caller. */
 }
