@@ -273,7 +273,7 @@ pagefunc_set_variable (char *tag, char *value)
   Package *orig_package = CurrentPackage;
 
   if (PageVars == (Package *)NULL)
-    PageVars = symbol_get_package (DEFAULT_PACKAGE_NAME);
+    PageVars = symbol_get_package_hash (DEFAULT_PACKAGE_NAME, 577);
 
   if (CurrentPackage == (Package *)NULL)
     symbol_set_default_package (PageVars);
@@ -289,7 +289,7 @@ pagefunc_set_variable_readonly (char *tag, char *value)
   Symbol *sym;
 
   if (PageVars == (Package *)NULL)
-    PageVars = symbol_get_package (DEFAULT_PACKAGE_NAME);
+    PageVars = symbol_get_package_hash (DEFAULT_PACKAGE_NAME, 577);
 
   if (CurrentPackage == (Package *)NULL)
     symbol_set_default_package (PageVars);
@@ -642,7 +642,7 @@ pagefunc_snarf_vars (char *string, int allow_assignments_p)
       }
 
       free (name);
-      if (value) free (value);
+      xfree (value);
     }
 
   return (package);
@@ -657,7 +657,7 @@ pagefunc_get_descriptor (char *tag)
 
   if (mhtml_function_package == (Package *)NULL)
     {
-      mhtml_function_package = symbol_get_package ("*meta-html*");
+      mhtml_function_package = symbol_get_package_hash ("*meta-html*", 577);
       initialize_external_functions (mhtml_function_package);
     }
 
@@ -726,7 +726,7 @@ alist_to_package (char *string)
 
 	  tag = STRING_VALUE (CAR (pair));
 
-	  if (!package)
+	  if (package == (Package *)NULL)
 	    {
 	      int old_prime = symbol_small_prime;
 	      symbol_small_prime = 23;
@@ -1194,6 +1194,7 @@ page_process_page_internal (PAGE *page)
 		      allow_keywords = (uf->flags & user_ACCEPT_KEYWORDS);
 		    /* else
 		      allow_keywords = (desc->flags & user_ACCEPT_KEYWORDS); */
+
 		    vars = pagefunc_snarf_vars (open_body, allow_keywords);
 		  }
 
@@ -1261,8 +1262,8 @@ page_process_page_internal (PAGE *page)
 		  else
 		    {
 		      char *display_body = (char *)NULL;
-		      PAGE *body;
-		      body = page_create_page ();
+		      PAGE *body = page_create_page ();
+
 		      page_set_contents (body, strbody);
 
 		      /* This text is no longer in the page. */
@@ -1430,7 +1431,7 @@ mhtml_evaluate_string (char *body)
     page_process_page_internal (evaluated);
     parser_current_lineno = lineno;
   }
-    
+
   result = evaluated->buffer;
   free (evaluated);
 
@@ -1593,7 +1594,8 @@ mhtml_add_user_function (int type, char *name, char *body, Package *vars)
       uf->body = strdup (body ? body : "");
 
       if (mhtml_user_keywords == (Package *)NULL)
-	mhtml_user_keywords = symbol_get_package ("*user-functions*");
+	mhtml_user_keywords =
+	  symbol_get_package_hash ("*user-functions*", 577);
       sym = symbol_intern_in_package (mhtml_user_keywords, name);
       sym->values = (char **)uf;
       sym->type = symtype_USERFUN;
@@ -2183,8 +2185,10 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 		  Package *local = param_package;
 		  char *x;
 
-		  if (local == (Package *)NULL)
+		  if (param_package == (Package *)NULL)
 		    local = symbol_get_package ((char *)NULL);
+		  else
+		    local = param_package;
 
 		  symbol_push_package (local);
 
@@ -2203,6 +2207,9 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 		  xfree (x);
 		  symbol_pop_package ();
 
+		  if (param_package == (Package *)NULL)
+		    symbol_destroy_package (local);
+
 		  if (savepack)
 		    {
 		      current_package = symbol_get_package (savepack);
@@ -2271,7 +2278,7 @@ mhtml_canonicalize_file_name (char *input, char *docroot, char *relpref,
   if (!relpref) relpref = "";
 
   if (maxlen >= workbuff_len)
-    workbuff = (char *)xrealloc (workbuff, workbuff_len = maxlen);
+    workbuff = (char *)xrealloc (workbuff, workbuff_len = maxlen + 10);
 
   /* Ignore leading and trailing whitespace. */
   input = strdup (input);
@@ -2328,10 +2335,12 @@ mhtml_canonicalize_file_name (char *input, char *docroot, char *relpref,
       }
   }
 
-  /* Save the web relative pathname here. */
-  if (saved_webname_len + 2 < strlen (workbuff))
+  /* Save the web relative pathname here.  The reason it is 20 bytes
+     longer is that require_search() likes to tack on an extension to
+     the web-relative name.  Always make sure there is enough room. */
+  if ((strlen (workbuff) + 20) > saved_webname_len)
     saved_webname = (char *)xrealloc
-      (saved_webname, (saved_webname_len = 10 + strlen (workbuff)));
+      (saved_webname, (saved_webname_len = 20 + strlen (workbuff)));
 
   strcpy (saved_webname, workbuff);
   *web_relative_name = saved_webname;
