@@ -28,8 +28,8 @@
    Santa Barbara, CA
    93101  */
 
-#if !defined (_PARSER_H_)
-#define _PARSER_H_
+#if !defined (_LIBMHTML_PARSER_H_)
+#define _LIBMHTML_PARSER_H_
 
 #define MHTML_INCLUDE_IS_RELATIVE 1
 #define MHTML_INCLUDE_RECURSION_LIMIT 10
@@ -83,6 +83,9 @@ extern UserFunction *mhtml_find_user_function (char *name);
 #define return_sequence(c, c1) \
      (((c) == '\n') || (((c) == '\r') && ((c1) == '\n')))
 
+#undef make_gcable
+#define make_gcable forms_gc_remember
+
 #if defined (LIBMHTML_INTERNAL)
 #if !defined (PACKAGE_INITIALIZER_EXTRA_CODE)
 #  define PACKAGE_INITIALIZER_EXTRA_CODE
@@ -104,7 +107,12 @@ initializer_function_name (Package *package)				\
    PACKAGE_INITIALIZER_EXTRA_CODE					\
 }
 
+/* The following defines are all used as markers for the `createdoc' program.
+   A couple of them produce output in the C source file in which they appear,
+   most notably the commands which define a function or macro. */
 #define DOC_SECTION(secname)
+#define DEFINE_SECTION(secname, keywords, short_info, long_info)
+#define DEFVAR(name, documentation)
 #define DEFUN(name, args, documentation) static void name (PFunArgs)
 #define DEFUNX(name, args, documentation)
 #define DEFMACRO(name, args, documentation) static void name (PFunArgs)
@@ -196,6 +204,7 @@ extern int mhtml_parser_interrupted;
 #define user_WHITESPACE_DELETED	0x01
 #define user_WEAK_MACRO		0x02
 #define user_ACCEPT_KEYWORDS	0x04
+#define user_WHITESPACE_KEPT	0x08
 
 extern char *metahtml_copyright_string;
 
@@ -240,6 +249,9 @@ extern char *get_value (Package *package, char *name);
 extern char **get_vars_names (Package *package);
 extern char **get_vars_vals (Package *package);
 extern void pagefunc_set_variable (char *tag, char *value);
+extern void mhtml_set_numeric_variable (char *name, int value);
+extern void mhtml_set_numeric_variable_in_package (Package *p,
+						   char *name, int value);
 extern void pagefunc_set_variable_readonly (char *tag, char *value);
 extern char *pagefunc_get_variable (char *tag);
 extern char *get_one_of (Package *package, char *tag, ...);
@@ -284,8 +296,8 @@ extern void mhtml_add_user_function (int type, char *name, char *body,
 
 /* Canonicalize the filename INPUT such that it is a complete and
    valid path to a file. */
-extern char *mhtml_canonicalize_file_name (char *input, char *incpref,
-					   char *relpref);
+extern char *mhtml_canonicalize_file_name
+	(char *input, char *incpref, char *relpref, char **web_relative_name);
 
 /* Same, but auto-handle the include and relative prefixes dependent on the
    setting of isp::web-relative-pathnames.  Also evaluates ARGUMENT, so you
@@ -314,9 +326,13 @@ extern char *mhtml_base64encode (char *data, int length, int shortlines);
 
 extern PAGE *parser_top_page (void);
 extern int number_p (char *string);
+extern int float_p (char *string);
+extern int integer_p (char *string, int base);
+
+extern PAGE *get_after_page_return_buffer (void);
 
 #if defined (__cplusplus)
 }
 #endif
 
-#endif /*_PARSER_H_ */
+#endif /*_LIBMHTML_PARSER_H_ */
