@@ -65,7 +65,7 @@ static PFunDesc func_table[] =
   { "%%FUNCTION-ALIST",		0, 0, pf_function_alist },
   { "%%QUOTE-FOR-SET-VAR",	0, 0, pf_quote_for_setvar },
   { "%%AFTER-PAGE-RETURN",	1, 0, pf_after_page_return },
-  { "APPLY",			0, 0, pf_apply },
+  { "APPLY",			-1, 0, pf_apply },
   { "%%BOOTSTRAP-METAHTML",	0, 0, pf_bootstrap_metahtml },
 
   { (char *)NULL,		0, 0, (PFunHandler *)NULL }
@@ -285,13 +285,14 @@ pf_read_package_file (PFunArgs)
       bprintf_insert (page, start, "%d", packages_read);
 
       if (mhtml_user_keywords == (Package *)NULL)
-	mhtml_user_keywords = symbol_lookup_package ("*user-functions*");
+	mhtml_user_keywords = 
+	  symbol_get_package_hash ("*user-functions*", 577);
     }
 }
 
 DEFUNX (pf_%%switch_user, newuser,
 "Changes the current user to <var newuser>, and returns \"true\" if the
-switch was successful.  This can only take place if the running Meta-HTML is
+switch was successful.  This can only take place if <Meta-HTML> is
 running under the user ID of the superuser (root).  After calling this
 function, it is impossible to switch back to another user, so it really
 isn't of much use in Meta-HTML programs.")
@@ -614,13 +615,28 @@ get_after_page_return_buffer (void)
       
 DEFUN (pf_apply, func &rest args,
 "Apply <var func> to <var args>.
-Examples:
+
+This <i>weak</i>macro can either be used as a simple tag or as a
+complex tag -- its usage is dependent on the function being called.
+
+Using <tag apply> as a simple tag:
 <example>
 <apply add 3 4 5>         --> 12
 <defun foo &key bar baz>
   <get-var bar>, <get-var baz>
 </defun>
 <apply foo \"bar=this baz=2\"> --> this,2
+</example>
+
+Using <tag apply> as a complex tag:
+<example>
+<defmacro upcase-text &key bold? &rest body>
+   <if <get-var-once bold?> <b>>
+   <upcase %qbody>
+   <if <get-var-once bold?> </b>>
+</defmacro>
+<apply upcase-text> This is a list of things to \"change case of\" </apply>
+<apply upcase-text bold?=true> And this is upcased <i>and</i> bold </apply>
 </example>")
 {
   char *func = mhtml_evaluate_string (get_positional_arg (vars, 0));
@@ -631,6 +647,20 @@ Examples:
       BPRINTF_BUFFER *funcall = bprintf_create_buffer ();
       int arg_index = 1;
       char *raw_arg;
+      UserFunction *uf = mhtml_find_user_function (func);
+      PFunDesc *desc = pagefunc_get_descriptor (func);
+      int macro_call = 0;
+
+      if (uf != (UserFunction *)NULL)
+	{
+	  if (uf->type == user_MACRO)
+	    macro_call = 1;
+	}
+      else if (desc != (PFunDesc *)NULL)
+	{
+	  if (desc->complexp != 0)
+	    macro_call = 1;
+	}
 
       bprintf (funcall, "<%s", func);
 
@@ -676,6 +706,13 @@ Examples:
 
       bprintf (funcall, ">");
 
+      /* If this is a macro call, we must print the body here. */
+      if (macro_call)
+	{
+	  bprintf (funcall, "%s", body->buffer);
+	  bprintf (funcall, "</%s>", func);
+	}
+
       if (debug_level > 5)
 	page_debug ("Apply: [%s]", funcall->buffer);
 
