@@ -6,10 +6,19 @@
 #if !defined (_PROFILER_H_)
 #define _PROFILER_H_
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 typedef struct
 {
   long times_called;	/* Number of times this function has been invoked. */
   double usecs_spent;	/* Number of microseconds spent in this function. */
 } PROFILE_INFO;
 
+#if defined (__cplusplus)
+}
+#endif
+
 #endif
