@@ -298,11 +298,11 @@ array variable <varref mhtml::require-loaded>.")
 		require_remember_pathname (pathname);
 		while ((pack = symbol_load_package (fd)) != (Package *)NULL);
 
+		close (fd);
+
 		if (mhtml_user_keywords == (Package *)NULL)
 		  mhtml_user_keywords = 
-		    symbol_lookup_package ("*user-functions*");
-
-		close (fd);
+		    symbol_get_package_hash ("*user-functions*", 577);
 
 		if ((uf = mhtml_find_user_function (".libinit"))
 		    != (UserFunction *)NULL)
