@@ -23,6 +23,23 @@ often the case, it isn't always convenient to re-invent the wheel, and
 so we have provided several tags in <Meta-HTML> PRO which deliver common
 functionality in a general way.", "")
 
+DEFINE_SECTION (USING-GNUPLOT, plotting; graphing; dynamic images,
+"<Meta-HTML> contains a few functions which make it easy to generate plots
+and graphs of data that you have available to you.
+
+The use of the functions requires `gnuplot', a copy-lefted graphing program
+widely available on the Internet (e.g., ftp://prep.ai.mit.edu/pub/gnu).
+
+Using the <funref using-gnuplot gnuplot::plot-alists> function, you can create
+PostScript and/or GIF images of a single set of data, or overlay many data
+sets in a single graph.
+
+Some terminology common to <code>gnuplot</code> may facilitate the reading
+of the function documentation:  a <i>tic</i> is a mark which appears
+on the axes of the final graph indicating the value along that axis --
+there are tics in both the <var x> and <var y> directions.  Tics can
+be labeled, you will have to read the documentation of
+<code>gnuplot</code> for more information on that flexible format.", "")
 
 DEFINE_SECTION (PAGECOUNT-APPENDIX, page counters; html helpers, 
 "The PAGECOUNT::... functions allow you to place English language or
@@ -42,6 +59,18 @@ functionality to the PowerStrip system itself, or who need to
 manipulate the underlying data structures and mechanisms of that
 system.  ", "")
 
+DEFINE_SECTION (AUTHORIZATION-TAGS, authorization;authentication;basic auth,
+"<Meta-HTML> provides several flexible methods of authenticating a user
+against a database.
+
+The functions defined in this section all work using HTTP's Basic
+Authorization scheme, but in a way that provides a much higher granularity
+than that of older directory based schemes.
+
+Because of the dynamic nature of <Meta-HTML>'s method, you can have many files
+in a single directory, some of which are authenticated against a user database,
+some of which are authenticated against a Unix style password file, or against
+an SQL database, and some of which do not require authentication at all.", "")
 
 DEFINE_SECTION (SERVER-VARIABLES, server; mhttpd; variables,
 "The <Meta-HTML> server (<code>mhttpd</code>) is configured by reading a
@@ -64,20 +93,6 @@ associated with the connecting browser.  This only has a minor effect
 on server performance, but isn't especially useful for non-<Meta-HTML>
 sites.")
 
-DEFVAR (ENV::EXEC-PATH, "The list of directories that <funref
-file-operators cgi-exec> will search when looking up the location of a
-command.
-
-Each component is a path to a directory, without the final trailing
-slash, separated from its neighbors by a colon character
-(<code>:</code>).
-
-For example:
-
-<example>
-   <set-var env::path=\"/www/bin:/www/docs/cgi-bin\">
-</example>")
-
 DEFVAR (ENV::HTTP_AUTHORIZATION, "The <code>Basic Authorization</code>
 string, as received by the server.  By the time that you see this
 string, the user has already been authorized.")
@@ -197,6 +212,10 @@ server itself.
 <get-var mhtml::debug-log>
 </complete-example>")
 
+DEFVAR (MHTML::DEBUG-POST, "When MHTML::DEBUG-LOG is set to a filename, and
+this variable is non-empty, the names and values of variables that are posted
+to the page are written to the debug log.")
+
 DEFVAR (MHTML::DEFAULT-FILENAMES, "An array specifying the file names
 that should be looked up when the URL to fetch ends in a directory,
 instead of a document specifier.  The files are searched for in the
