@@ -44,6 +44,11 @@
    times out. */
 #define SESSION_TIMEOUT 61
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 /* The underlying data type of a unique session identifier. */
 typedef char *session_id_t;
 
@@ -121,4 +126,9 @@ extern SESSION_INFO **session_all_sessions (void);
    Returns the number of sessions reaped. */
 extern int session_reap (void);
 
+#if defined (__cplusplus)
+}
+#endif
+
 #endif /* !_SESSION_H_ */
+
