@@ -14,6 +14,106 @@ DEFUN (pf_avg,  &rest nums[],
  <avg 4 9 3.2>  --> 5.40
  </example>
 ")
+DOC_SECTION (AUTHORIZATION-TAGS)
+DEFUN (pf_basic_auth::authorization_database,  filename type,
+" When passed FILENAME, sets the default authorization database
+ file to be that fully qualified pathname.  When passed an empty
+ FILENAME, returns the current pathname of the authorization
+ database.
+
+ The default authorization database used is the standard Meta-HTML
+ PowerStrip database.  Note that authorizing based on this method
+ does not log the user into the PowerStrip system, it simply
+ authenticates them against that database.
+
+ If <var type> is set to \"Meta-HTML\" (the default), then the
+ database is assumed to be a standard Meta-HTML/GDBM database,
+ with a key value of the username, and a field called PASSWORD.
+ The password field will be decoded as a unix based password.
+
+ If <var type> is set to \"Unix\", then the authorization database
+ is treated as if it were a standard Unix password file (/etc/passwd).
+
+ If <var type> is set to anything else, then it is assumed to be one
+ of the valid SQL database types for your Meta-HTML installation, and
+ <var filename> is treated as the <b>DSN</b> to use to talk to that
+ database.  The table which is searched is called <b>AUTH</b>, and it
+ must contain <b>username</b> and <b>password</b> fields.
+")
+DOC_SECTION (AUTHORIZATION-TAGS)
+DEFUN (pf_basic_auth::force_login_popup,  realm page-var,
+" Force the appearance of a popup dialog on the user's screen which asks
+ for a username and password.
+
+ <var realm> is a string which is displayed at the top of the popup
+ dialog -- it is the variable text in \"Enter username for <var realm>\".
+
+ <var page-var> is the name of a binary variable which contains the
+ contents of the page to display in case the user clicks the \"Cancel\"
+ button in the dialog box.
+")
+DOC_SECTION (AUTHORIZATION-TAGS)
+DEFUN (pf_basic_auth::gdbm_authorize,  username password,
+" Authorize <var username> and <var password> against the GDBM style
+ database specified in <var basic-auth::authorization-database>.
+
+ The database must contain fields of <code>username</code>, and
+ <code>password</code>.  The password field should be encrypted
+ in the fashion normal for that system (i.e., unix::crypt).
+
+ Returns \"true\" if the authentication succeeds, otherwise the empty
+ string.
+")
+DOC_SECTION (AUTHORIZATION-TAGS)
+DEFUN (pf_basic_auth::get_auth_info, ,
+" Retrieves the authorization info passed to the server from the browser.
+ Places the base 64 decoded username in <var basic-auth::username> and
+ the base 64 decoded password in <var basic-auth::password>.
+
+ You probably don't want to use this function directly.  See
+ <funref AUTHORIZATION-TAGS basic-auth::require-authorization> instead.
+")
+DOC_SECTION (POWERSTRIP-SYSTEM-TAGS)
+DEFUN (pf_basic_auth::powerstrip_login, ,
+"")
+DOC_SECTION (AUTHORIZATION-TAGS)
+DEFUN (pf_basic_auth::require_authorization,  for-realm bad-login-page,
+" Force the connecting browser to be authenticated against the local
+ authentication database or display a popup window asking the user
+ for a username and password.
+
+ <var for-realm> is a string to print at the top of the popup.
+
+ <var  bad-login-page> is the name of a page to redirect to when
+ authorization has completely failed.
+
+ The entire authorization step is bypassed if the variable
+ <b><code>basic-auth::skip-auth</code></b> is set to a non-empty value.
+
+ Please read the description of <funref authorization-tags
+ basic-auth::authorization-database> for more details.
+")
+DOC_SECTION (AUTHORIZATION-TAGS)
+DEFUN (pf_basic_auth::sql_authorize,  username password,
+" Authorize <var username> and <var password> against the SQL style
+ database whose DSN is in <var basic-auth::authorization-database>.
+
+ The database must contain a table called <code>AUTH</code>, and that
+ table must have fields of <code>username</code>, and
+ <code>password</code>.
+
+ Returns \"true\" if the authentication succeeds, otherwise the empty
+ string.
+")
+DOC_SECTION (AUTHORIZATION-TAGS)
+DEFUN (pf_basic_auth::unix_authorize,  username password,
+" Authorize <var username> and <var password> against the Unix style
+ password file in <var basic-auth::authorization-database> (usually,
+ /etc/passwd).
+
+ Returns \"true\" if the authentication succeeds, otherwise the empty
+ string.
+")
 DOC_SECTION (LANGUAGE-OPERATORS)
 DEFUN (pf_bootstrapper::system_initialize, ,
 " Defined in libmhtml/standard.mhtml, this function is called immediately
@@ -345,6 +445,22 @@ DEFUN (pf_dir::canonicalize_pathname,  path,
  </complete-example>
 ")
 DOC_SECTION (FILE-OPERATORS)
+DEFUN (pf_dir::create_emacs_backup,  pathname &key max force,
+" Create an Emacs-style backup of <var pathname>, and return the name
+ of the backup file.  No more than <var max> backups may exist --
+ if you pass <var max> as a positive integer then multiple backups
+ up to that many will be made.  The default value for <var max> is
+ infinity.
+
+ If <var pathname> exists, a backup of it is made by appending
+ \"~<i>num</i>\" to the name.  If the most recent backup is within
+ 1 hour of the existing file, no backup is made (unless <var force>
+ is \"true\". Thus, we maintain some compatibility with the Emacs
+ style of making backup files.
+
+ If <var pathname> is already a backup file, nothing is done.
+")
+DOC_SECTION (FILE-OPERATORS)
 DEFUN (pf_dir::dirname,  pathname,
 " Return the directory part of <var pathname>.  This simply
  returns everything but the last component of <var pathname>.
@@ -449,7 +565,7 @@ DOC_SECTION (NETWORK-APPENDIX)
 DEFUN (pf_e_mail::parse_message,  msgvar,
 " Given a complete RFC-822 E-mail message stored in <var msgvar>, this
  function returns an alist describing the message in detail.
-;
+
  The elements of the alist include the raw headers, and variables
  called <var hdr_TO>, <var hdr_FROM>, <var hdr_SUBJECT>, <var hdr_CC>,
  and <var hdr_DATE>, in additions to the variable <var body>.
@@ -478,6 +594,15 @@ DEFUN (pf_e_mail::sendmail,  from to[] cc[] bcc[] headers[] message,
  conversation that takes place between <Meta-HTML> and the SMTP
  server at <var e-mail::mailhost>.
 ")
+DOC_SECTION (LANGUAGE-OPERATORS)
+DEFUN (pf_eval_file,  filename &optional &unevalled alt,
+" Read the contents of <var filename>, and evaluate its contents in the
+ context of the caller.  If <var filename> cannot be read, then execute
+ the value of <var alt> if it is passed.
+
+ <var filename> is a fully qualified pathname to a file anywhere within
+ the file system.
+")
 DOC_SECTIONS (ARITHMETIC-OPERATORS)
 DEFUN (pf_factorial,  x,
 " Computes the factorial of <var x>.
@@ -533,10 +658,9 @@ DEFMACRO (pf_form_to_database,  database &key dbname key redirect-to action-name
  Defaults to \"Submit Form\".
 </ul>
 ")
-DOC_SECTION (MISCELLANEOUS-TAGS)
+DOC_SECTION (HTML-HELPERS)
 DEFUN (pf_forms::multiple_instance_resolve,  session-pack dest-pack,
-" DOC-SECTION (HTML-HELPERS)
- For every instance of a form in SESSION-PACK, append the
+" For every instance of a form in SESSION-PACK, append the
  elements in order to arrays in DEST-PACK.  Thus, if your
  form contained \"name\" and \"age\" input fields, and the user
  created multiple instances of this form, the output of
@@ -580,6 +704,91 @@ DOC_SECTION (MISCELLANEOUS-TAGS)
 DEFUN (pf_function_def,  fun,
 " Return a human readable rendition of the function named by <var fun>.
 ")
+DOC_SECTION (USING-GNUPLOT)
+DEFUN (pf_gnuplot::make_gif_plot,  data &optional output-file &key style xlabel ylabel title xtics ytics,
+" Create a GIF plot of the points in <var data>, using <var style>
+ (default 'boxes') and perhaps labelling the X and Y axes.
+
+ The output GIF will be stored in <var output-file>, or returned
+ directly to <code>*standard-output*</code>, (in which case, the
+ output stream is then closed).
+
+ The default value for <var title> is \"Unspecified Data Graph\".
+ 
+ Possibilities for <var style> are:
+ <code>lines</code>, <code>points</code>, <code>linespoints</code>,
+ <code>impulses</code>, <code>dots</code>, <code>steps</code>,
+ <code>fsteps</code>, <code>histeps</code>, <code>errorbars</code>,
+ <code>xerrorbars</code>, <code>yerrorbars</code>,
+ <code>xyerrorbars</code>, <code>boxes</code>, <code>boxerrorbars</code>,
+ <code>boxxyerrorbars</code>, <code>financebars</code>,
+ <code>candlestick</code>s or <code>vector</code>. 
+
+")
+DOC_SECTION (USING-GNUPLOT)
+DEFUN (pf_gnuplot::make_ps_plot,  data &key style xlabel ylabel title xtics ytics xtics-rotate,
+" Create a PostScript plot of the points in <var data>, using <var style>
+ (default 'boxes') and perhaps labelling the X and Y axes.
+
+ The default value for <var title> is \"Unspecified Data Graph\".
+ 
+ This function returns the PostScript text necessary to produce
+ the graph itself -- the text could be passed to a PostScript
+ printer, or, one could use
+ <funreg using-gnuplot gnuplot::postscript-to-gif> to
+ create a GIF image suitable for displaying on a Web page.
+
+ Possibilities for <var style> are:
+ <code>lines</code>, <code>points</code>, <code>linespoints</code>,
+ <code>impulses</code>, <code>dots</code>, <code>steps</code>,
+ <code>fsteps</code>, <code>histeps</code>, <code>errorbars</code>,
+ <code>xerrorbars</code>, <code>yerrorbars</code>,
+ <code>xyerrorbars</code>, <code>boxes</code>, <code>boxerrorbars</code>,
+ <code>boxxyerrorbars</code>, <code>financebars</code>,
+ <code>candlestick</code>s or <code>vector</code>. 
+
+ <example>
+ <set-var data = \"20 45 40 47 68 98\">
+ <set-var ps = <gnuplot::make-ps-plot <get-var-once data[]> title=\"Foo\">>
+ </example>
+
+")
+DOC_SECTION (USING-GNUPLOT)
+DEFUN (pf_gnuplot::plot_alist_internal,  stream alist,
+" You pass in <var stream> and an <var alist> containing:
+ <ul>
+    <li>DATA-FUNCTION: The name of the function which generates the points
+<li>DATA-FUNCTION-ARGS: Arguments to pass to DATA-FUNCTION
+		  <li>TITLE: A title for this data (defaults to DATA-FUNCTION).
+             <li>STYLE: The style for plotting.  See (gnuplot::make-plot).
+             <li>SCALE: A factor for scaling the data.
+ </ul>
+")
+DOC_SECTION (USING-GNUPLOT)
+DEFUN (pf_gnuplot::plot_alists,  &key output-type varname output-file rotation xlabel ylabel xtics ytics xtics-rotate boxwidth &rest alists[],
+" For now, see <funref using-gnuplot gnuplot::plot-alist-internal>.
+
+ Call `gnuplot' to actually generate the plot from the data.
+ The default output type is GIF (works with gnuplot 3.6 and greater).
+")
+DOC_SECTION (USING-GNUPLOT)
+DEFUN (pf_gnuplot::postscript_to_gif,  ps-data &key filename varname,
+" Using the PostScript text in <var ps-data>, create a GIF image
+ suitable for display from a Web page.
+
+ If <var filename=foo.gif> is specified, then the GIF image will
+ be written to \"foo.gif\", and the function returns \"true\" if successful.
+
+ If <var varname=gif-var> is specified, then the GIF image will
+ be placed into the binary variable \"GIF-VAR\", and the function returns
+ \"true\" if successful.
+
+ If neither <var filename> nor <var varname> is specified, the
+ function writes the raw data of the GIF image as an HTML
+ document to the standard output, and closes the standard output
+ stream.  This is to facilitate the creation of Web pages which
+ produce GIF images as output.
+")
 DOC_SECTION (NETWORK-APPENDIX)
 DEFUN (pf_http::get_document,  host port path &key strip-headers var &rest headers[],
 " Get an HTTP document specified by <var host>, <var port> and <var path>.
@@ -673,7 +882,7 @@ DEFUN (pf_incdir, ,
 DOC_SECTION (MISCELLANEOUS-TAGS)
 DEFUN (pf_lang,  &key en de fr it es,
 " Provides an interface for selecting one of several phrases
- in different languages, based on the value of AUTHDATA::LANGUAGE.
+ in different languages, based on the value of <var authdata::language>.
 
  Use in the following situations:
  <example code>
@@ -683,12 +892,12 @@ DEFUN (pf_lang,  &key en de fr it es,
 ")
 DOC_SECTION (MISCELLANEOUS-TAGS)
 DEFUN (pf_lang_inform,  &key en de fr it es &rest info,
-" This function uses the value of authdata::language to select a
+" This function uses the value of <var authdata::language> to select a
  language, and then calls <var inform> with <info>.
 ")
 DOC_SECTION (MISCELLANEOUS-TAGS)
 DEFUN (pf_lang_message,  &key en de fr it es &rest message,
-" This function uses the value of authdata::language to select a
+" This function uses the value of <var authdata::language> to select a
  language, and then calls <var message> with that choice.
 ")
 DOC_SECTION (HTML-HELPERS)
@@ -804,7 +1013,7 @@ DEFUN (pf_mailable_address,  addr,
 " Return \"true\" if ADDR appears to be a correctly formatted E-mail address.
 ")
 DOC_SECTION (PACKAGES)
-DEFUN (pf_make_alist,  &rest args,
+DEFUN (pf_make_alist, ,
 " Return an alist from the name=value pairs passed.
  <complete-example>
  <make-alist foo=bar baz=this>
@@ -943,9 +1152,8 @@ DEFUN (pf_pagecount::gif_counter,  fontname,
 DOC_SECTION (DATES-AND-TIMES)
 DEFUN (pf_parse_date_string,  string,
 " Deprecated.  Please use <funref DATES-AND-TIMES date::parse-date> instead.
- <example>
- <parse-date-string> is equivalent to <date::parse-date <date> date>
- </example>
+ <example code> <parse-date-string> </example> is equivalent to
+ <example code> <date::parse-date <date> date> </example>
 ")
 DOC_SECTION (FILE-OPERATORS)
 DEFUN (pf_path::find_executable,  basename,
@@ -959,11 +1167,18 @@ DEFUN (pf_path::find_executable,  basename,
 ")
 DOC_SECTION (ARITMETIC-OPERATORS)
 DEFUN (pf_percent,  num total,
-" Return the percentage of TOTAL that NUM is.
+" Return the percentage of <var total> that <var num> is.
  <example>
  <percent 20 200> --> 10.00
  </example>
 ")
+DOC_SECTION (ARITMETIC-OPERATORS)
+DEFUN (pf_percentage,  percent total,
+" Return the number which is <var percentage> of <var total>.
+ <example>
+ <percentage 10 200> --> 20.00
+ </example>
+")
 DOC_SECTION (MACRO-COMMANDS)
 DEFUN (pf_primitive?,  name,
 " Returns \"true\" if <var name> is defined as a <Meta-HTML> primitive, or
@@ -974,7 +1189,7 @@ DEFUN (pf_select_option,  varname display &optional value,
 " Use instead of the <example code><option></example> HTML tag within
  the select HTML tag.
 
- Creates a <i>option</i> item in the page in which <var varname>
+ Creates an <i>option</i> item in the page in which <var varname>
  is expected to contain the current value of the choice, or nothing,
  if no choice has been made.  If you supply the optional <var value>
  argument, that value is returned instead of <var display>.
@@ -1000,7 +1215,7 @@ DEFUN (pf_select_option_value,  varname display &optional value,
 " Use instead of the <example code><option></example> HTML tag within
  the select HTML tag.
 
- Creates a <i>option</i> item in the page in which <var varname>
+ Creates an <i>option</i> item in the page in which <var varname>
  is expected to contain the current value of the choice, or nothing,
  if no choice has been made.  If you supply the optional <var value>
  argument, that value is returned instead of <var display>.
@@ -1022,9 +1237,11 @@ DEFUN (pf_select_option_value,  varname display &optional value,
 </example>
 ")
 DOC_SECTION (SESSION-OPERATORS)
-DEFUN (pf_session_destroy_package,  package,
-" Delete all of the variables referenced by <var package> in the
- current session.  Also see <funref packages session-export>.
+DEFUN (pf_session_destroy_package,  &rest packages,
+" Delete all of the variables in the current session which are prefixed
+ by any of the packages specified in <var packages>.
+
+ Also see <funref packages session-export>.
 ")
 DOC_SECTION (SESSION-OPERATORS)
 DEFUN (pf_session::initialize,  &optional timeout timeout-page,
@@ -1566,11 +1783,15 @@ DEFUN (pf_sql::standard_search_form,  &optional package &key fsize href title,
 
  When this form is posted to, it gathers all of the
  columns which match the specified search criteria, and then
- displayes the results in a table, linking each to the
+ displays the results in a table, linking each to the
  sql::search-editurl.  When only one result is found, redirects
  to the page specified in <code>search::editurl</code>, providing a
  message in the session, displayable with
  <funref html-helpers show-message>, which explains why.
+
+ For more information on what happens when this form is posted to,
+ please see <funref generic-sql-interface
+ sql::standard-search-form-handler>.
 ")
 DOC_SECTION (GENERIC-SQL-INTERFACE)
 DEFUN (pf_sql::standard_search_handler,  package &key fsize href,
@@ -1646,15 +1867,7 @@ DEFUN (pf_string_to_array,  string arrayvar,
 ")
 DOC_SECTION (STRING-OPERATORS)
 DEFUN (pf_strings::both_trim,  varname,
-" Trims whitespace from both the \"left\" and \"right\" -hand sides
- of the string stored in <var varname>, replacing the contents of
- that variable with the trimmed string.
- <complete-example>
- <set-var foo=\"    string with whitespace on left and right     \">
- String: [<get-var-once foo>]
- <strings::both-trim foo>
- String: [<get-var-once foo>]
- </complete-example>
+" Obsoleted by <funref string-operators strings::trim>.
 ")
 DOC_SECTION (STRING-OPERATORS)
 DEFUN (pf_strings::left_trim,  varname,
@@ -1688,7 +1901,7 @@ DEFUN (pf_strings::trim,  varname,
  <complete-example>
  <set-var foo=\"    string with whitespace on left and right     \">
  String: [<get-var-once foo>]
- <strings::both-trim foo>
+ <strings::trim foo>
  String: [<get-var-once foo>]
  </complete-example>
 ")
