@@ -935,8 +935,9 @@ DEFUN (pf_stream_put, stream string,
 }
 
 DEFUN (pf_stream_put_contents, stream var,
-"Writes the contents of the binary variable <var var> to the open <var
-stream>.  Binary variables are generally created with <funref
+"Writes the contents of the variable <var var> to the open <var
+stream>.  This is the only way to get the contents of a binary variable
+written to a stream.  Binary variables are generally created with <funref
 stream-operators stream-get-contents>, or when uploading a file from
 a Web browser.")
 {
