@@ -123,7 +123,7 @@ DEFUN (pf_string_length, string,
 
 #define MAX_SUBEXPS 10
 DEFUN (pf_match, string regex
-       &key action=[delete | extract | report | startpos | endpos | length],
+       &key action=[delete|extract|report|startpos|endpos|length],
 "Matches <var regexp> against <var string>, and then performs the
 indicated <var action>.  The default for <var action> is \"report\".
 
@@ -139,7 +139,7 @@ When action is \"endpos\", returns the numeric offset of the end of the
 matched substring.
 
 <var regexp> is an extended Unix regular expression, the complete syntax of
-which is beyone the scope of this document.  However, the essential
+which is beyond the scope of this document.  However, the essential
 basics are:
 <ul>
 <li> A period (<code>.</code>) matches any one character.
@@ -918,6 +918,13 @@ comparison.
   char *caseless = mhtml_evaluate_string (get_value (vars, "caseless"));
   int caseless_p = 0;
 
+  if (debug_level > 10)
+    {
+      page_debug ("[ arg1 = '%s', \n  arg2 = '%s' ]",
+		  arg1 ? arg1 : "",
+		  arg2 ? arg2 : "");
+    }
+		  
   if (!empty_string_p (caseless))
     caseless_p++;
 
