@@ -76,7 +76,11 @@ static int AP_slots = 0;
 
 /* Default number of slots for a symbol table.  Some callers
    may want to change this if they create a large number of
-   symbol tables designed to hold only a few entries. */
+   symbol tables designed to hold only a few entries.
+
+   Our standard value here is 107, which is good when there are a lot
+   of relatively small tables.  Other reasonable primes are:
+   29, 227, 307, 577, 797, 991, 1013, 3347, and 4999. */
 int symbol_small_prime = 107;
 
 /* The package which is "current".  That is, this is the package to use
@@ -361,21 +365,24 @@ package_pdl_remove (Package *package)
 
 /* Create a package named NAME suitable for storing symbols in.
    If the package already exists, return that package.
+   A non-zero value for SMALL_PRIME creates the table with that many
+   hash buckets, instead of using the default value.
    You can create an anonymous package by passing NULL as the name. */
 Package *
-symbol_get_package (char *name)
+symbol_get_package_hash (char *name, int small_prime)
 {
   register int i = 0;
   Package *package = (Package *)NULL;
 
-  package = symbol_lookup_package (name);
+  if (name != (char *)NULL)
+    package = symbol_lookup_package (name);
 
   if (package == (Package *)NULL)
     {
       package = (Package *)xmalloc (sizeof (Package));
       package->name = name ? strdup (name) : name;
       package->name_len = name ? strlen (name) : 0;
-      package->table = symbol_create_table (0);
+      package->table = symbol_create_table (small_prime);
 
       if ((AP_index + 2) > AP_slots)
 	AllPackages = (Package **) xrealloc
@@ -392,6 +399,15 @@ symbol_get_package (char *name)
   return (package);
 }
 
+/* Create a package named NAME suitable for storing symbols in.
+   If the package already exists, return that package.
+   You can create an anonymous package by passing NULL as the name. */
+Package *
+symbol_get_package (char *name)
+{
+  return (symbol_get_package_hash (name, 0));
+}
+
 /* Destroy the package PACKAGE, freeing all of the space that it
    was using.  Delete the package from the list of AllPackages. */
 void
@@ -410,7 +426,8 @@ symbol_destroy_package (Package *package)
 	    if (package == AllPackages[i])
 	      {
 		/* Move subsequent packages back. */
-		while ((AllPackages[i] = AllPackages[i+1]) != (Package *)NULL)
+		while ((AllPackages[i] = AllPackages[i + 1])
+		       != (Package *)NULL)
 		  i++;
 
 		AP_index--;
@@ -633,7 +650,7 @@ symbol_intern (char *name)
   if (package == (Package *)NULL)
     {
       if (!CurrentPackage)
-	CurrentPackage = symbol_get_package (DEFAULT_PACKAGE_NAME);
+	CurrentPackage = symbol_get_package_hash (DEFAULT_PACKAGE_NAME, 577);
 
       package = CurrentPackage;
     }
@@ -656,7 +673,7 @@ symbol_lookup (char *name)
      no package part.  In that case, create the default package. */
   if (package == (Package *)NULL)
     {
-      package = symbol_get_package (DEFAULT_PACKAGE_NAME);
+      package = symbol_get_package_hash (DEFAULT_PACKAGE_NAME, 577);
       CurrentPackage = package;
     }
 
@@ -679,7 +696,7 @@ symbol_get_values (char *name)
      no package part.  In that case, create the default package. */
   if (package == (Package *)NULL)
     {
-      package = symbol_get_package (DEFAULT_PACKAGE_NAME);
+      package = symbol_get_package_hash (DEFAULT_PACKAGE_NAME, 577);
       CurrentPackage = package;
     }
 
@@ -717,7 +734,7 @@ symbol_package_symbols (char *name)
   if (name)
     {
       if (name[0] == '\0')
-	package = symbol_get_package (DEFAULT_PACKAGE_NAME);
+	package = symbol_get_package_hash (DEFAULT_PACKAGE_NAME, 577);
       else
 	package = symbol_get_package (name);
     }
@@ -1093,8 +1110,9 @@ symbol_add_value (Symbol *symbol, char *value)
 char *
 symbol_full_name (Symbol *symbol)
 {
-  char *name;
+  char *sname = symbol->preserved_name ? symbol->preserved_name : symbol->name;
   Package *pack = (Package *)symbol->package;
+  char *name;
 
   if (pack->name != (char *)NULL)
     {
@@ -1102,18 +1120,10 @@ symbol_full_name (Symbol *symbol)
       strcpy (name, pack->name);
       name[pack->name_len + 0] = ':';
       name[pack->name_len + 1] = ':';
-      if (symbol->preserved_name)
-	strcpy (name + pack->name_len + 2, symbol->preserved_name);
-      else
-	strcpy (name + pack->name_len + 2, symbol->name);
+      strcpy (name + pack->name_len + 2, sname);
     }
   else
-    {
-      if (symbol->preserved_name)
-	name = strdup (symbol->preserved_name);
-      else
-	name = strdup (symbol->name);
-    }
+    name = strdup (sname);
 
   return (name);
 }
