@@ -136,6 +136,13 @@ extern Package *symbol_lookup_package (char *name);
    You can create an anonymous package by passing NULL as the name. */
 extern Package *symbol_get_package (char *name);
 
+/* Create a package named NAME suitable for storing symbols in.
+   If the package already exists, return that package.
+   A non-zero value for SMALL_PRIME creates the table with that many
+   hash buckets, instead of using the default value.
+   You can create an anonymous package by passing NULL as the name. */
+extern Package *symbol_get_package_hash (char *name, int small_prime);
+
 /* Push CurrentPackage onto the Package PDL, and make NEW_PACKAGE 
    the current package. */
 extern void symbol_push_package (Package *new_package);
