@@ -233,17 +233,14 @@ mhtml::relative-prefix> among others.")
 
 static void
 get_var_internal (PFunArgs, int once)
-  {
+{
   register int i;
   char *name;
 
   for (i = 0; (name = get_positional_arg (vars, i)) != (char *)NULL; i++)
     {
-      char *insertion;
-      char *value;
-
-      insertion = mhtml_evaluate_string (name);
-      value = pagefunc_get_variable (insertion);
+      char *insertion = mhtml_evaluate_string (name);
+      char *value = pagefunc_get_variable (insertion);
 
       if (debug_level > 5)
 	page_debug ("<get-var \"%s\">", insertion ? insertion : "");
@@ -264,8 +261,7 @@ get_var_internal (PFunArgs, int once)
       if (debug_level > 5)
 	page_debug ("--> `%s'", value ? value : "");
 
-      if (insertion)
-	free (insertion);
+      xfree (insertion);
 
       if (once)
 	*newstart = start;
@@ -336,7 +332,7 @@ Example:
   for (i = 0; (name = get_positional_arg (vars, i)) != (char *)NULL; i++)
     {
       char *varname = mhtml_evaluate_string (name);
-      Symbol *sym = varname ?  symbol_lookup (varname) : (Symbol *)NULL;
+      Symbol *sym = varname ? symbol_lookup (varname) : (Symbol *)NULL;
 
       if (sym)
 	{
@@ -369,9 +365,12 @@ Example:
 
 DEFUN (pf_var_exists, name,
 "<code>var-exists</code> checks for the <i>existence</i> of
-the variable named by <var varname>, and returns <code>true</code> if that variable exists.
+the variable named by <var varname>, and returns <code>true</code> if that
+variable exists.
 
-The existence of a variable has nothing to do with its value -- a variable exists if it has been created with <funref variables set-var>, and doesn't exist after it has been unset with <funref variables unset-var>.
+The existence of a variable has nothing to do with its value -- a variable
+exists if it has been created with <funref variables set-var>, and doesn't
+exist after it has been unset with <funref variables unset-var>.
 
 <example>
   <set-var foo=1 bar>
@@ -449,7 +448,8 @@ Also see <funref variables decrement>.")
 }
 
 DEFUN (pf_decrement, name &key by=amount,
-"Subtract <var amount> (default 1) from the contents of the variable named by <var varname>.
+"Subtract <var amount> (default 1) from the contents of the variable named
+by <var varname>.
 
 <example>
    <set-var foo=1>
@@ -702,7 +702,9 @@ The size value is zero for all other variable types.")
 DEFUN (pf_copy_var, from-var &optional to-var...,
 "Copies the variable <var from-var> to each of the named <var to-var>s.
 
-Each <var to-var> becomes the repository of a copy of the information already stored under <var from-var>.  This is a <i>true</i> copy; not an alias to the original variable.
+Each <var to-var> becomes the repository of a copy of the information
+already stored under <var from-var>.  This is a <i>true</i> copy; not
+an alias to the original variable.
 
 <example>
 <set-var foo=bar>
