@@ -742,7 +742,7 @@ mr_set_cookie (HTTP_RESULT *result, char *ignore)
 	      if (timeout > 0)
 		{
 		  if (permanent_p)
-		    ticks = 946684799;
+		    ticks = 1293825599;	/* Fri Dec 31 11:59:59 2010 */
 		  else
 		    ticks += timeout;
 		}
@@ -1085,6 +1085,72 @@ mhttpd_fill_in_missing_headers (HTTP_RESULT *result)
 #endif
 }
 
+void
+mhttpd_debug_request (HTTP_REQUEST *req)
+{
+  register int i;
+  BPRINTF_BUFFER *b = bprintf_create_buffer ();
+
+  bprintf (b, "Request:\n");
+
+  bprintf (b, "struct HTTP_REQUEST *%0xl = {\n", (unsigned long) req);
+  bprintf (b, "           char *method = %s\n", req->method);
+  bprintf (b, "         char *location = %s\n", req->location);
+  bprintf (b, "         char *protocol = %s\n", req->protocol);
+  bprintf (b, " char *protocol_version = %s\n", req->protocol_version);
+  bprintf (b, "        char *requester = %s\n", req->requester);
+  bprintf (b, "   char *requester_addr = %s\n", req->requester_addr);
+  bprintf (b, "              int flags = (");
+  if (req->flags == 0) bprintf (b, "NONE");
+  if (req->flags & flag_KEEP_ALIVE) bprintf (b, "KEEP-ALIVE");
+  bprintf (b, ")\n");
+
+  bprintf (b, "  MIME_HEADER **headers = {\n");
+
+  for (i = 0; req->headers && req->headers[i]; i++)
+    bprintf (b, "     %s: %s\n", req->headers[i]->tag, req->headers[i]->value);
+
+  bprintf (b, "  }\n}\n");
+
+  mhttpd_log (log_DEBUG, "%s", b->buffer);
+  bprintf_free_buffer (b);
+}
+
+void
+mhttpd_debug_doc_spec (DOC_SPEC *spec)
+{
+  BPRINTF_BUFFER *b = bprintf_create_buffer ();
+
+  bprintf (b, "DOC_SPEC:\n");
+
+  bprintf (b, "struct DOC_SPEC *%0xl = {\n", (unsigned long) spec);
+  bprintf (b, "    char *requested_path = %s\n", spec->requested_path);
+  bprintf (b, "     char *physical_path = %s\n", spec->physical_path);
+  bprintf (b, "      char *logical_path = %s\n", spec->logical_path);
+  bprintf (b, "         char *path_info = %s\n", spec->path_info);
+  bprintf (b, "      char *query_string = %s\n", spec->query_string);
+  bprintf (b, "         char *mime_type = %s\n", spec->mime_type);
+  bprintf (b, "            int doc_type = ");
+
+  {
+    char *dtype = (char *)NULL;
+
+    switch (spec->doc_type)
+      {
+      case doc_NONE: dtype = "NONE"; break;
+      case doc_PARSEABLE_MHTML: dtype = "PARSEABLE_MHTML"; break;
+      case doc_EXTERNAL_CGI: dtype = "EXTERNAL_CGI"; break;
+      case doc_BASIC: dtype = "BASIC"; break;
+      case doc_SERVER_REDIRECT: dtype = "SERVER_REDIRECT"; break;
+      }
+
+    bprintf (b, "%s\n", dtype);
+  }
+  bprintf (b, "}\n");
+  mhttpd_log (log_DEBUG, "%s", b->buffer);
+  bprintf_free_buffer (b);
+}
+
 static HTTP_RESULT *
 mhtml_make_result (void)
 {
@@ -1182,6 +1248,9 @@ mhttpd_build_environment (HTTP_RESULT *result)
 
       sprintf (lbuff, "%d", spec->content_length);
       add_to_env ("CONTENT_LENGTH", lbuff);
+      if (mhttpd_get_mime_header (headers, "Content-type"))
+	add_to_env ("CONTENT_TYPE",
+		    mhttpd_get_mime_header (headers, "Content-type"));
     }
 
   add_to_env ("SCRIPT_NAME", spec->argv[0]);
@@ -1302,7 +1371,7 @@ mhtml_post_to_document (char *string)
 	    char *debug_p = pagefunc_get_variable ("mhtml::debug-post");
 	    int debug = 0;
 
-	    if (!empty_string_p (debug_p)) debug=1;
+	    if (!empty_string_p (debug_p)) debug = 1;
 
 	    for (i = 0; symbols[i] != (Symbol *)NULL; i++)
 	      {
@@ -2332,6 +2401,10 @@ http_get_handler (ReqFunArgs)
 
   result->request = request;
   result->spec = mhttpd_resolve_location (request);
+
+  if (mhttpd_debugging)
+    mhttpd_debug_doc_spec (result->spec);
+
   mhtml_read_content_from_fd (result, mhtml_stdin_fileno);
 
   if (result->spec != (DOC_SPEC *)NULL)
