@@ -699,7 +699,7 @@ read_character ()
 
   if ((the_input_string != (char *)NULL) &&
       (the_input_string[the_input_string_index] != '\0'))
-    character = the_input_string[the_input_string_index++];
+    character = the_input_string[the_input_string_index++] & 0x00ff;
 
   return (character);
 }
