@@ -38,32 +38,34 @@ extern "C"
 
 typedef enum { LT_string, LT_number, LT_cons, LT_symbol, LT_nil } WispType;
 
+typedef struct WISPOBJECT WispObject;
 typedef struct
 {
-  void *car;
-  void *cdr;
+  WispObject *car;
+  WispObject *cdr;
 } Cons;
 
 typedef struct
 {
   char *pname;
-  void *value;
+  WispObject *value;
 } WispSymbol;
 
-typedef struct _wisp_object_
+struct WISPOBJECT
 {
   WispType type;
-  union {
+  union
+  {
     char *string;
     double number;
     Cons cons;
     WispSymbol *symbol;
   } val;
-} WispObject;
+};
 
 /* Here reside some macros to make life more palatable. */
-#define CAR(obj) ((WispObject *)((obj)->val.cons.car))
-#define CDR(obj) ((WispObject *)((obj)->val.cons.cdr))
+#define CAR(obj) ((obj)->val.cons.car)
+#define CDR(obj) ((obj)->val.cons.cdr)
 #define CADR(obj) CAR (CDR (obj))
 #define CDAR(obj) CDR (CAR (obj))
 #define CDDR(obj) CDR (CDR (obj))
@@ -73,7 +75,7 @@ typedef struct _wisp_object_
 #define NUMBER_VALUE(obj)  ((obj)->val.number)
 #define STRING_LENGTH(obj) (strlen (STRING_VALUE (obj)))
 #define SYMBOL_PNAME(obj)  ((obj)->val.symbol->pname)
-#define SYMBOL_VALUE(obj)  ((WispObject *)((obj)->val.symbol->value))
+#define SYMBOL_VALUE(obj)  ((obj)->val.symbol->value)
 #define SETQ(symbol_object, value) (SYMBOL_VALUE (symbol_object) = value)
 
 /* Macros which check the type. */
