@@ -8,6 +8,11 @@
 
 #include <stdlib.h>
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 static void memory_error_and_abort (char *fname);
 
 /* **************************************************************** */
@@ -50,4 +55,9 @@ memory_error_and_abort (char *fname)
   fprintf (stderr, "%s: Out of virtual memory!\n", fname);
   abort ();
 }
+#if defined (__cplusplus)
+}
+#endif
+
 #endif /* !ALREADY_HAVE_XMALLOC */
+
