@@ -1597,7 +1597,8 @@ mdb_apropos (MDBArgs)
 			  else if (pack == mhtml_user_keywords)
 			    bprintf (buffer, " Function: <%s>\n", symname);
 			  else
-			    bprintf (buffer, "%s::%s\n", pack->name, symname);
+			    bprintf (buffer, " Variable: %s::%s\n",
+				     pack->name, symname);
 			}
 		    }
 		}
