@@ -425,6 +425,11 @@ initialize_mdb (int argc, char *argv[])
 			    pagefunc_set_variable
 			      ("mhtml::relative-prefix", orig_dir + tlen);
 			}
+
+		      set_session_database_location
+			(pagefunc_get_variable
+			 ("mhttpd::session-database-file"));
+
 		      free (orig_dir);
 		    }
 		  else
