@@ -3,6 +3,11 @@
 /*  Copyright (c) 1995 Brian J. Fox
     Author: Brian J. Fox (bfox@ai.mit.edu) Mon Oct  9 06:09:41 1995.  */
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 extern void _rl_redisplay_after_sigwinch (void);
 extern void _rl_update_final (void);
 extern void _rl_erase_at_end_of_line (int l);
@@ -16,13 +21,17 @@ extern int rl_character_len (int c, int pos);
 extern int rl_show_char (int c);
 extern void _rl_move_vert (int to);
 
-/* Move the cursor from _rl_last_c_pos to NEW, which are buffer indices.
+/* Move the cursor from _rl_last_c_pos to NEWPOS, which are buffer indices.
    DATA is the contents of the screen line of interest; i.e., where
    the movement is being done. */
-extern void _rl_move_cursor_relative (int new, char *data);
+extern void _rl_move_cursor_relative (int newpos, char *data);
 
 /* Actually update the display, period. */
 extern void rl_forced_update_display (void);
 extern void rl_on_new_line (void);
 extern void rl_redisplay (void);
 extern int rl_expand_prompt (char *prompt);
+
+#if defined (__cplusplus)
+}
+#endif
