@@ -42,6 +42,11 @@
 #include "memalloc.h"
 #include "history.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 /* Not all systems declare ERRNO in errno.h... and some systems #define it! */
 #if !defined (errno)
 extern int errno;
@@ -1125,17 +1130,18 @@ get_subst_pattern (char *str, int *iptr, int delimiter, int is_rhs,
 static void
 postproc_subst_rhs (void)
 {
-  char *new;
+  char *replacement;
   int i, j, new_size;
 
-  new = xmalloc (new_size = subst_rhs_len + subst_lhs_len);
+  replacement = xmalloc (new_size = subst_rhs_len + subst_lhs_len);
   for (i = j = 0; i < subst_rhs_len; i++)
     {
       if (subst_rhs[i] == '&')
 	{
 	  if (j + subst_lhs_len >= new_size)
-	    new = xrealloc (new, (new_size = new_size * 2 + subst_lhs_len));
-	  strcpy (new + j, subst_lhs);
+	    replacement = xrealloc
+	      (replacement, (new_size = new_size * 2 + subst_lhs_len));
+	  strcpy (replacement + j, subst_lhs);
 	  j += subst_lhs_len;
 	}
       else
@@ -1144,13 +1150,14 @@ postproc_subst_rhs (void)
 	  if (subst_rhs[i] == '\\' && subst_rhs[i + 1] == '&')
 	    i++;
 	  if (j >= new_size)
-	    new = xrealloc (new, new_size *= 2);
-	  new[j++] = subst_rhs[i];
+	    replacement = xrealloc (replacement, new_size *= 2);
+	  replacement[j++] = subst_rhs[i];
 	}
     }
-  new[j] = '\0';
+
+  replacement[j] = '\0';
   free (subst_rhs);
-  subst_rhs = new;
+  subst_rhs = replacement;
   subst_rhs_len = j;
 }
 
@@ -2144,9 +2151,14 @@ main ()
 }
 
 #endif /* TEST */
+#if defined (__cplusplus)
+}
+#endif
+
 
 /*
 * Local variables:
 * compile-command: "gcc -g -DTEST -o history history.c"
 * end:
 */
+
