@@ -40,6 +40,10 @@
 #define STREQ(a, b)	(((a)[0] == (b)[0]) && (strcmp ((a), (b)) == 0))
 #define STREQN(a, b, n)	(((a)[0] == (b)[0]) && (strncmp ((a), (b), (n)) == 0))
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 /* Variables imported from other files in the readline library. */
 extern Keymap _rl_keymap;
 extern HIST_ENTRY *saved_line_for_history;
@@ -373,3 +377,7 @@ rl_search_history (int direction, int invoking_key)
 
   return 0;
 }
+
+#if defined (__cplusplus)
+}
+#endif
