@@ -39,6 +39,11 @@
 
 #define abs(x)		(((x) > 0) ? (x) : -(x))
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 extern char *xmalloc (int), *xrealloc (void *, int);
 extern void rl_extend_line_buffer (int len);
 
@@ -361,3 +366,7 @@ rl_history_search_backward (int count, int ignore)
     prev_line_found = (char *)NULL;
   return (rl_history_search_internal (abs (count), (count > 0) ? -1 : 1));
 }
+
+#if defined (__cplusplus)
+}
+#endif
