@@ -2,6 +2,12 @@
 
 /*  Copyright (c) 1995 Brian J. Fox
     Author: Brian J. Fox (bfox@ai.mit.edu) Mon Oct  9 08:15:45 1995.  */
+
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 extern char *tgetstr (char *id, char **area);
 extern char *tgoto (char *cm, int destcol, int destline);
 extern int tputs (char *cp, int affcnt, int (*outc)(int));
@@ -10,3 +16,7 @@ extern int tgetnum (char *id);
 extern int tgetflag (char *id);
 extern char PC;
 extern char *BC, *UP;
+
+#if defined (__cplusplus)
+}
+#endif
