@@ -32,6 +32,11 @@
 #include <pwd.h>
 #include "tilde.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 #if defined (USG) && !defined (HAVE_GETPW_DECLS)
 extern struct passwd *getpwuid (), *getpwnam ();
 #endif /* USG && !defined (HAVE_GETPW_DECLS) */
@@ -362,3 +367,8 @@ memory_error_and_abort ()
  * end:
  */
 #endif /* TEST */
+
+#if defined (__cplusplus)
+}
+#endif
+
