@@ -29,7 +29,7 @@
     Santa Barbara, CA
     93101  */
 
-#include "language.h"
+#include "modules.h"
 
 #if defined (__cplusplus)
 extern "C"
@@ -44,30 +44,13 @@ static PFunDesc ftab[] =
   { (char *)NULL,		0,	 0,		(PFunHandler *)NULL }
 };
 
-void
-module_initialize (void)
-{
-  static int called  = 0;
+MODULE_INITIALIZE ("csv", ftab)
 
-  if (!called)
-    {
-      register int i;
-      Symbol *sym, *funcnames;
-
-      funcnames = symbol_intern ("modules::syms-of-csv");
-
-      /* Install the names and pointers. */
-      for (i = 0; ftab[i].tag != (char *)NULL; i++)
-	{
-	  sym = symbol_intern_in_package (mhtml_function_package, ftab[i].tag);
-	  symbol_add_value (funcnames, ftab[i].tag);
-	  sym->type = symtype_FUNCTION;
-	  sym->values = (char **)(&ftab[i]);
-	}
-    }
-}
-
-void _init (void) { module_initialize (); }
+DEFINE_SECTION (CSV-MODULE, , 
+"The CSV module is useful for importing <b>C</b>omma <b>S</b>eperated
+<b>V</b>alues from external programs which are capable of producing them.
+Such programs include word processors, legacy database systems, spreadsheets,
+and the like.", "")
 
 /* Return an array of strings, each one representing data taken from the
    text in TEXT.  Each element of TEXT is separated from the next by
@@ -135,13 +118,6 @@ csv_to_array (char *text, char *sep)
   return (results);
 }
 
-DOC_SECTION (CSV-MODULE)
-DEFUNX (pf_csv_module_synopsis, ,
-"The CSV module is useful for importing <b>C</b>omma <b>S</b>eperated
-<b>V</b>alues from external programs which are capable of producing them.
-Such programs include word processors, legacy database systems, spreadsheets,
-and the like.")
-
 DEFUNX (pf_csv::csv_to_array, cvs-text array-name &key sep=SEPARATOR,
 "Returns an array of the values specified in the comma separated values
 represented by <var csv-text>.  The values are placed in <var array-name>
