@@ -29,12 +29,13 @@
     Santa Barbara, CA
     93101  */
 
-#include "language.h"
+#include "modules.h"
 
 #if defined (__cplusplus)
 extern "C"
 {
 #endif
+
 /* 0) #include any files that are specific to your module. */
 
 /* 1) Declare the functions which implement the Meta-HTML functionality. */
@@ -51,33 +52,7 @@ static PFunDesc ftab[] =
   { (char *)NULL,		0,	0,	(PFunHandler *)NULL }
 };
 
-/* 3) Write the function which installs the commands, and performs any other
-      module specific initializations.  This function is called exactly once.*/
-void
-module_initialize (void)
-{
-  static int called = 0;
-
-  if (!called)
-    {
-      register int i;
-      Symbol *sym, *funcnames;
-
-      called++;
-      funcnames = symbol_intern ("modules::syms-of-logs");
-
-      /* Install the names and pointers. */
-      for (i = 0; ftab[i].tag != (char *)NULL; i++)
-	{
-	  sym = symbol_intern_in_package (mhtml_function_package, ftab[i].tag);
-	  symbol_add_value (funcnames, ftab[i].tag);
-	  sym->type = symtype_FUNCTION;
-	  sym->values = (char **)(&ftab[i]);
-	}
-    }
-}
-
-void _init (void) { module_initialize (); }
+MODULE_INITIALIZE ("logs", ftab)
 
 /* 4) Write the actual code which implements your functionality. */
 
@@ -127,7 +102,6 @@ pf_parse_line (PFunArgs)
 	  }
       }
 
-
       /* User. */
       skip_whitespace (line, i);
       bprintf (output, "(\"user\" . \"");
@@ -201,20 +175,20 @@ pf_parse_line (PFunArgs)
       if (line[i]) i++;
 
       /* Request Type. */
-      bprintf (output, "(\"req-type\" . \"");
+      bprintf (output, "(\"req_type\" . \"");
       while (line[i] && line[i] != ' ') bprintf (output, "%c", line[i++]);
       bprintf (output, "\")");
 
 
       /* Request URL. */
       skip_whitespace (line, i);
-      bprintf (output, "(\"req-url\" . \"");
+      bprintf (output, "(\"req_url\" . \"");
       while (line[i] && line[i] != ' ') bprintf (output, "%c", line[i++]);
       bprintf (output, "\")");
 
       /* Request Protocol. */
       skip_whitespace (line, i);
-      bprintf (output, "(\"req-protocol\" . \"");
+      bprintf (output, "(\"req_protocol\" . \"");
       while (line[i] && line[i] != '"') bprintf (output, "%c", line[i++]);
       bprintf (output, "\")");
       if (line[i]) i++;
@@ -222,7 +196,7 @@ pf_parse_line (PFunArgs)
 
       /* Result Code. */
       skip_whitespace (line, i);
-      bprintf (output, "(\"result-code\" . \"");
+      bprintf (output, "(\"result_code\" . \"");
       while (line[i] && line[i] != ' ') bprintf (output, "%c", line[i++]);
       bprintf (output, "\")");
 
