@@ -334,6 +334,12 @@ index.html\nhome.html");
       pagefunc_set_variable ("mhtml::metahtml-extensions[]", ".mhtml");
     }
 
+  if (forced_port_number)
+    mhtml_set_numeric_variable ("mhtml::forced-server-port",
+				forced_port_number);
+  if (debug_mode)
+    pagefunc_set_variable ("mhtml::debug-mode", "true");
+
   if (page->buffer)
     page_process_page (page);
 
@@ -1226,281 +1232,292 @@ loop_over_connections (void)
 
 	  /* Okay, read the HTTP request and handle it. */
 	  req = http_read_request (connection);
+
 	  mhtml_stdout_fileno = connection;
 	  mhtml_stdin_fileno = connection;
 	  mhtml_stderr_fileno = connection;
 
+	  if (req != (HTTP_REQUEST *)NULL)
+	    {
 #if defined (USE_KEEP_ALIVE)
-	  if (req && kept_alive)
-	    req->flags |= flag_KEEP_ALIVE;
+	      if (kept_alive)
+		req->flags |= flag_KEEP_ALIVE;
 #endif /* USE_KEEP_ALIVE */
 
-	  if (!debug_mode && !know_my_client)
-	    {
-#if defined (MHTTPD_DNS_CACHE)
-	      char dns_cached_result[256];
-	      int cache_found = 0;
-#endif
-	      struct sockaddr_in client;
-
-	      if (getpeername
-		  (connection, (struct sockaddr *)&client, &sock_size) != -1)
+	      if (!debug_mode && !know_my_client)
 		{
-		  struct hostent *client_info = (struct hostent *)NULL;
-		  char *client_info_name = (char *)NULL;
-		  long addr =  client.sin_addr.s_addr;
-		  const char *x = (const char *)&addr;
-		  char *addr_rep = (char *)xmalloc (16);
-		  char *get_remote_host =
-		    pagefunc_get_variable ("mhttpd::get-remote-host");
-
-		  sprintf (addr_rep, "%d.%d.%d.%d",
-			   (unsigned char)x[0],
-			   (unsigned char)x[1],
-			   (unsigned char)x[2],
-			   (unsigned char)x[3]);
-
-		  req->requester_addr = addr_rep;
-
-		  if ((get_remote_host == (char *)NULL) ||
-		      (strcasecmp (get_remote_host, "true") == 0))
-		    {
 #if defined (MHTTPD_DNS_CACHE)
-		      if (mhttpd_cache_dns)
+		  char dns_cached_result[256];
+		  int cache_found = 0;
+#endif
+		  struct sockaddr_in client;
+
+		  if (getpeername
+		      (connection,
+		       (struct sockaddr *)&client,
+		       &sock_size) != -1)
+		    {
+		      struct hostent *client_info = (struct hostent *)NULL;
+		      char *client_info_name = (char *)NULL;
+		      long addr =  client.sin_addr.s_addr;
+		      const char *x = (const char *)&addr;
+		      char *addr_rep = (char *)xmalloc (16);
+		      char *get_remote_host =
+			pagefunc_get_variable ("mhttpd::get-remote-host");
+
+		      sprintf (addr_rep, "%d.%d.%d.%d",
+			       (unsigned char)x[0],
+			       (unsigned char)x[1],
+			       (unsigned char)x[2],
+			       (unsigned char)x[3]);
+
+		      req->requester_addr = addr_rep;
+
+		      if ((get_remote_host == (char *)NULL) ||
+			  (strcasecmp (get_remote_host, "true") == 0))
 			{
-			  DBFILE db;
-
-			  db = database_open (MHTTPD_DNS_CACHE, DB_READER);
-
-			  if (db != (DBFILE)NULL)
+#if defined (MHTTPD_DNS_CACHE)
+			  if (mhttpd_cache_dns)
 			    {
-			      DBOBJ key, *val;
+			      DBFILE db;
 
-			      key.data =((unsigned char *)req->requester_addr);
-			      key.length = 1 + strlen ((char *)key.data);
-			      val = database_fetch (db, &key);
+			      db = database_open (MHTTPD_DNS_CACHE, DB_READER);
 
-			      if (val != (DBOBJ *)NULL)
+			      if (db != (DBFILE)NULL)
 				{
-				  if (val->data)
+				  DBOBJ key, *val;
+
+				  key.data =
+				    ((unsigned char *)req->requester_addr);
+				  key.length = 1 + strlen ((char *)key.data);
+				  val = database_fetch (db, &key);
+
+				  if (val != (DBOBJ *)NULL)
 				    {
-				      cache_found++;
-				      strncpy (dns_cached_result,
-					       (char *)val->data, val->length);
-				      dns_cached_result[val->length] = '\0';
-				      client_info_name = dns_cached_result;
-				      free (val->data);
+				      if (val->data)
+					{
+					  cache_found++;
+					  strncpy
+					    (dns_cached_result,
+					     (char *)val->data, val->length);
+					  dns_cached_result[val->length] ='\0';
+					  client_info_name = dns_cached_result;
+					  free (val->data);
+					}
+				      free (val);
 				    }
-				  free (val);
+				  database_close (db);
 				}
-			      database_close (db);
+			    }
+
+			  if (client_info_name == (char *)NULL)
+#endif /* MHTTPD_DNS_CACHE */
+			    {
+			      client_info = gethostbyaddr (x, 4, AF_INET);
+			      if (client_info)
+				client_info_name = (char *)client_info->h_name;
 			    }
 			}
 
-		      if (client_info_name == (char *)NULL)
-#endif /* MHTTPD_DNS_CACHE */
+		      if (client_info_name)
 			{
-			  client_info = gethostbyaddr (x, 4, AF_INET);
-			  if (client_info)
-			    client_info_name = (char *)client_info->h_name;
-			}
-		    }
+			  req->requester = strdup (client_info_name);
 
-		  if (client_info_name)
-		    {
-		      req->requester = strdup (client_info_name);
-
-		      /* Some sneaky bastards don't return a real hostname.
-			 When that happens, use the address instead of the
-			 name. */
-		      {
-			register int i, dots=0;
-
-			for (i = 0; req->requester[i] != '\0'; i++)
-			  if (req->requester[i] == '.')
-			    dots++;
-
-			if (dots < 2)
+			  /* Some sneaky bastards don't return a real hostname.
+			     When that happens, use the address instead of the
+			     name. */
 			  {
-			    free (req->requester);
-			    req->requester = strdup (addr_rep);
+			    register int i, dots=0;
+
+			    for (i = 0; req->requester[i] != '\0'; i++)
+			      if (req->requester[i] == '.')
+				dots++;
+
+			    if (dots < 2)
+			      {
+				free (req->requester);
+				req->requester = strdup (addr_rep);
+			      }
 			  }
-		      }
-		    }
-		  else
-		    req->requester = strdup (addr_rep);
+			}
+		      else
+			req->requester = strdup (addr_rep);
 
 #if defined (MHTTPD_DNS_CACHE)
-		  if (mhttpd_cache_dns && !cache_found)
-		    {
-		      /* I don't really want to save the data in this
-			 process, but I can't see a relatively inexpensive
-			 way to do it otherwise. */
-		      DBFILE db;
-
-		      db = database_open (MHTTPD_DNS_CACHE, DB_WRCREAT);
-		      if (db != (DBFILE)NULL)
+		      if (mhttpd_cache_dns && !cache_found)
 			{
-			  DBOBJ key, val;
-			  key.data = (unsigned char *)req->requester_addr;
-			  val.data = (unsigned char *)client_info_name;
-			  key.length = 1 + strlen ((char *)key.data);
-			  val.length = 1 + strlen ((char *)val.data);
-			  database_store (db, &key, &val);
-			  database_close (db);
+			  /* I don't really want to save the data in this
+			     process, but I can't see a relatively inexpensive
+			     way to do it otherwise. */
+			  DBFILE db;
+
+			  db = database_open (MHTTPD_DNS_CACHE, DB_WRCREAT);
+			  if (db != (DBFILE)NULL)
+			    {
+			      DBOBJ key, val;
+			      key.data = (unsigned char *)req->requester_addr;
+			      val.data = (unsigned char *)client_info_name;
+			      key.length = 1 + strlen ((char *)key.data);
+			      val.length = 1 + strlen ((char *)val.data);
+			      database_store (db, &key, &val);
+			      database_close (db);
+			    }
 			}
-		    }
 #endif /* MHTTPD_DNS_CACHE */
 
 #if defined (I_UNDERSTAND_IDENT)
-		  /* Do the IDENT lookup if that is requested. */
-		  if (pagefunc_get_variable ("mhttpd::get-remote-ident"))
-		    {
-		      int fd = -1;
+		      /* Do the IDENT lookup if that is requested. */
+		      if (pagefunc_get_variable ("mhttpd::get-remote-ident"))
+			{
+			  int fd = -1;
 
-		      mhttpd_log (log_DEBUG, "Trying to get ident from %s",
-				  req->requester);
+			  mhttpd_log (log_DEBUG, "Trying to get ident from %s",
+				      req->requester);
 
 #if defined (NOTDEF)
-		      fd = tcp_to_host (req->requester_addr, "113");
+			  fd = tcp_to_host (req->requester_addr, "113");
 #else
-		      /* Get a socket on our local machine which is
-			 using the same IP address as the server. */
-		      {
-			int ident_fd = socket (AF_INET, SOCK_STREAM, 0);
-			struct sockaddr_in local, remote;
-			int one = 1;
-
-			memset (&local, 0, sizeof (local));
-			memset (&remote, 0, sizeof (remote));
-			local.sin_addr = host_address;
-			remote.sin_addr = client.sin_addr;
-			local.sin_family = AF_INET;
-			remote.sin_family = AF_INET;
-			remote.sin_port = htons (113);
-
-			if (ident_fd > 0)
+			  /* Get a socket on our local machine which is
+			     using the same IP address as the server. */
 			  {
-			    int error = 0;
-			    setsockopt (ident_fd, SOL_SOCKET, SO_REUSEADDR,
-					(char *)&one, sizeof (one));
+			    int ident_fd = socket (AF_INET, SOCK_STREAM, 0);
+			    struct sockaddr_in local, remote;
+			    int one = 1;
 
-			    /* Bind the socket to our address. */
-			    error = bind (ident_fd, (struct sockaddr *)&local,
-					  sizeof (local));
-			    if (error != -1)
+			    memset (&local, 0, sizeof (local));
+			    memset (&remote, 0, sizeof (remote));
+			    local.sin_addr = host_address;
+			    remote.sin_addr = client.sin_addr;
+			    local.sin_family = AF_INET;
+			    remote.sin_family = AF_INET;
+			    remote.sin_port = htons (113);
+
+			    if (ident_fd > 0)
 			      {
-				signal (SIGALRM, (sig_t)connect_timed_out);
-				alarm (10);
-				error = connect (ident_fd,
-						 (struct sockaddr *)&remote,
-						 sizeof (remote));
-				alarm (0);
-				signal (SIGALRM, SIG_DFL);
+				int error = 0;
+				setsockopt (ident_fd, SOL_SOCKET, SO_REUSEADDR,
+					    (char *)&one, sizeof (one));
 
+				/* Bind the socket to our address. */
+				error = bind
+				  (ident_fd,
+				   (struct sockaddr *)&local,
+				   sizeof (local));
 				if (error != -1)
-				  fd = ident_fd;
+				  {
+				    signal (SIGALRM, (sig_t)connect_timed_out);
+				    alarm (10);
+				    error = connect
+				      (ident_fd,
+				       (struct sockaddr *)&remote,
+				       sizeof (remote));
+				    alarm (0);
+				    signal (SIGALRM, SIG_DFL);
+
+				    if (error != -1)
+				      fd = ident_fd;
+				  }
 			      }
 			  }
-		      }
 #endif
-		      if (fd > -1)
-			{
-			  char comm[100];
-			  char *ident;
+			  if (fd > -1)
+			    {
+			      char comm[100];
+			      char *ident;
 
 #if defined (__WINNT__)
 #  define ntohs(x) x
 #endif
-			  sprintf (comm, "%u , %u\r\n",
-				   ntohs (client.sin_port), http_port);
-			  write (fd, comm, strlen (comm));
-			  lseek (fd, 0, SEEK_END);
-			  shutdown (fd, 1);
+			      sprintf (comm, "%u , %u\r\n",
+				       ntohs (client.sin_port), http_port);
+			      write (fd, comm, strlen (comm));
+			      lseek (fd, 0, SEEK_END);
+			      shutdown (fd, 1);
 
-			  if ((read (fd, comm, 99) > 0) &&
-			      ((ident = strstr (comm, "USERID :"))
-			       != (char *)NULL))
-			    {
-			      register int i;
+			      if ((read (fd, comm, 99) > 0) &&
+				  ((ident = strstr (comm, "USERID :"))
+				   != (char *)NULL))
+				{
+				  register int i;
 
-			      for (i = ident - comm; i < 100; i++)
-				if ((comm[i] == '\n') || (comm[i] == '\r'))
-				  {
-				    comm[i] = '\0';
-				    break;
-				  }
+				  for (i = ident - comm; i < 100; i++)
+				    if ((comm[i] == '\n') || (comm[i] == '\r'))
+				      {
+					comm[i] = '\0';
+					break;
+				      }
 
-			      while ((i--) > 0)
-				if (comm[i] == ':')
-				  {
-				    i++;
-				    while (comm[i] == ' ' && i < 100) i++;
-				    break;
-				  }
-			      pagefunc_set_variable_readonly
-				("mhtml::remote-ident", comm + i);
+				  while ((i--) > 0)
+				    if (comm[i] == ':')
+				      {
+					i++;
+					while (comm[i] == ' ' && i < 100) i++;
+					break;
+				      }
+				  pagefunc_set_variable_readonly
+				    ("mhtml::remote-ident", comm + i);
+				}
+			      mhttpd_log (log_DEBUG, "Got %s", comm);
+			      shutdown (fd, 2);
+			      close (fd);
 			    }
-			  mhttpd_log (log_DEBUG, "Got %s", comm);
-			  shutdown (fd, 2);
-			  close (fd);
 			}
-		    }
 #endif /* I_UNDERSTAND_IDENT */
+		    }
+		  know_my_client = 1;
 		}
-	      know_my_client = 1;
-	    }
 
-	  mhttpd_mime_headers_to_package
-	    (req->headers, "mhttpd-received-headers");
-	  pagefunc_set_variable ("mhttpd::method", req->method);
-	  pagefunc_set_variable ("mhttpd::protocol", req->protocol);
-	  pagefunc_set_variable ("mhttpd::protocol-version",
-				 req->protocol_version);
-	  pagefunc_set_variable ("mhttpd::location", req->location);
-	  pagefunc_set_variable ("mhttpd::requester", req->requester);
-	  pagefunc_set_variable ("mhttpd::requester-addr",
-				 req->requester_addr);
+	      mhttpd_mime_headers_to_package
+		(req->headers, "mhttpd-received-headers");
+	      pagefunc_set_variable ("mhttpd::method", req->method);
+	      pagefunc_set_variable ("mhttpd::protocol", req->protocol);
+	      pagefunc_set_variable ("mhttpd::protocol-version",
+				     req->protocol_version);
+	      pagefunc_set_variable ("mhttpd::location", req->location);
+	      pagefunc_set_variable ("mhttpd::requester", req->requester);
+	      pagefunc_set_variable ("mhttpd::requester-addr",
+				     req->requester_addr);
 
-	  if (req && mhttpd_per_request_function)
-	    {
-	      PAGE *perfun = page_create_page ();
-	      bprintf (perfun, "<%s>\n", mhttpd_per_request_function);
-	      page_process_page (perfun);
-
-	      if (mhttpd_page_redirect_p (perfun))
+	      if (mhttpd_per_request_function)
 		{
-		  if (!strncasecmp (perfun->buffer, "HTTP/", 5) == 0)
-		    bprintf_insert (perfun, 0, "HTTP/1.0 302 Found\n");
+		  PAGE *perfun = page_create_page ();
+		  bprintf (perfun, "<%s>\n", mhttpd_per_request_function);
+		  page_process_page (perfun);
 
-		  mhttpd_write (connection, perfun->buffer, perfun->bindex);
-		  goto exit_now;
+		  if (mhttpd_page_redirect_p (perfun))
+		    {
+		      if (!strncasecmp (perfun->buffer, "HTTP/", 5) == 0)
+			bprintf_insert (perfun, 0, "HTTP/1.0 302 Found\n");
+
+		      mhttpd_write (connection, perfun->buffer, perfun->bindex);
+		      goto exit_now;
+		    }
+
+		  if (perfun)
+		    page_free_page (perfun);
 		}
 
-	      if (perfun)
-		page_free_page (perfun);
-	    }
+	      if (mhttpd_debugging)
+		{
+		  register int i;
+		  BPRINTF_BUFFER *buffer = bprintf_create_buffer ();
 
-	  if (mhttpd_debugging && req)
-	    {
-	      register int i;
-	      BPRINTF_BUFFER *buffer = bprintf_create_buffer ();
+		  bprintf (buffer, "\nGot Request: `%s %s %s/%s' from %s:\n",
+			   req->method, req->location,
+			   req->protocol, req->protocol_version,
+			   req->requester);
 
-	      bprintf (buffer, "\nGot Request: `%s %s %s/%s' from %s:\n",
-		       req->method, req->location,
-		       req->protocol, req->protocol_version,
-		       req->requester);
+		  if (kept_alive)
+		    bprintf (buffer, "(Connection was Kept Alive)\n");
 
-	      if (kept_alive)
-		bprintf (buffer, "(Connection was Kept Alive)\n");
+		  bprintf (buffer, "Headers Received:\n");
 
-	      bprintf (buffer, "Headers Received:\n");
-
-	      for (i = 0; req->headers && req->headers[i]; i++)
-		bprintf (buffer, "   %s: %s\n",
-			 req->headers[i]->tag, req->headers[i]->value);
-	      mhttpd_log (log_DEBUG, "%s", buffer->buffer);
-	      bprintf_free_buffer (buffer);
+		  for (i = 0; req->headers && req->headers[i]; i++)
+		    bprintf (buffer, "   %s: %s\n",
+			     req->headers[i]->tag, req->headers[i]->value);
+		  mhttpd_log (log_DEBUG, "%s", buffer->buffer);
+		  bprintf_free_buffer (buffer);
+		}
 	    }
 
 	  kept_alive = 0;
